/**
 * @include "../../../xui/sys.jsp"
 * @include "../../../xui/util.js"
 * @include "../../../xui/uibase.js"
 * @include "../../../xui/xwindow.js"
 * @include "../../../xui/ctrls/xdialog.js"
 * @type 
 */
/**
 * 该JS包含各页面常用的和界面无关的JS对象，所有对象都寄存在根页面，可以通过getServer()访问该js下的所有对象。
 */
var LOGINTYPE_NONE = -1;// 未登录
var LOGINTYPE_ADMIN = 1;// 是服务器管理员
var LOGINTYPE_USER = 2; // 是服务器用户

/**
 * LoginId 对象在客户端完全没有必要，其功能完全可以由Login对象来代替。 
 */

/*登陆对象,从server对象可以获得此对象得一个唯一实例*/
function Login(detail) {
	this._ppt = new Map();
	this._userprops = new UserProperties();
	this._loadServerLoginInfo(detail);
}

/*取得用户自定义属性*/
Login.prototype.getUserProperty = function(key, def) {
	var v = this._userprops.get(key);
	return v ? v : def;
}

Login.prototype.getUserProperties = function() {
	return this._userprops;
}

/*设置用户自定义属性*/
Login.prototype.setUserProperty = function(key, val) {
	this._userprops.set(key, val);
}

/**
 * 删除指定的用户自定义属性
 * @param {} key
 */
Login.prototype.removeUserProperty = function(key) {
	this._userprops.remove(key);
}

/*装入能访问的资源，和其他信息*/
Login.prototype._loadServerLoginInfo = function(detail) {
	if (!detail) {
		detail = queryObj("js/login.do", 'action=getlogininfo');
	}
	this._loginInfo(detail);
}

Login.prototype._loginInfo = function(detail) {
	var txtload = new TxtLoader(detail);
	// 装入属性
	var propertySection = txtload.getSection("property");
	this._loadProperty(propertySection);
	// 载入用户自定义属性
	var userProps = txtload.getSection("userproperties");
	this._loadUserProperties(userProps);
}

// 载入用户自定义属性
Login.prototype._loadUserProperties = function(section) {
	if (!section) {
		return;
	}
	var content = section.getContentStr();
	this._userprops.loadFromString(content);
}

/*装入属性,包含当前报表期，登陆用户所属的部门*/
Login.prototype._loadProperty = function(section) {
	if (!section) {
		//throwError("不能读取 登陆用户的信息");
		throwError(I18N.getString("ebi.user.main.objsatroot.js.str1","不能读取 登陆用户的信息"));
	}
	var detail = section.getContentStr();
	var mp = new StringMap(detail, "\r\n");
	this.type = mp.getInt('loginType', -1);
	if (!this.isLogined())
		return;
	this.id = mp.getValue('id', '');
	this.loginName = mp.getValue('name', '');
	this.orgid = mp.getValue('orgid', '');
}

/*获取权限判断接口*/
Login.prototype.getPmChecker = function() {
	if (this.pmchecker)
		return this.pmchecker;
	if (typeof(PmChecker) == 'undefined')
		sys.lib.include('ebi/js/pmchecker.js');
	this.pmchecker = new PmChecker(this);
	return this.pmchecker;
}

Login.prototype.getProperty = function(key, def) {
	var r = this._ppt.get(key);
	return r ? r : def;
}

/**
 * 去掉客户端关于vfs文件的处理，vfs文件的处理，通过服务器端得HomeDir完成。 
 */

/*
通过js设置当前用户的Session值，例如，所有的表都有一个默认的参数，@param， 该值是与登陆用户相关的，那么我们就可以在门户里面调用该函数，设置 session的值。
ps:session中如果含有@大头的属性，BI就认为其是参数。
设置当前登陆用户的sesseion的值，多用于设置登陆用户所使用的默认参数*/
Login.prototype.setSession = function(name, value) {
	var map = new Map();
	map.put("action", "set");
	map.put("obj", "session");
	map.put("name", name);
	map.put("value", value);
	var dtl = queryObj("js/reqmgr.do", map);
}

/*
proxyBbhId=用户指定代理的报表户单位
*/
Login.prototype.setProperty = function(key, value) {
	this._ppt.put(key, value);
}

/*是否是管理员*/
Login.prototype.isAdmin = function() {
	return this.type == LOGINTYPE_ADMIN;
}

/*是否是服务器用户*/
Login.prototype.isUser = function() {
	return this.type == LOGINTYPE_USER;
}

/*判断是否登录了*/
Login.prototype.isLogined = function() {
	return this.type != LOGINTYPE_NONE;
}

/**
 * 返回登录者的账号
 * @return {}
 */
Login.prototype.getId = function() {
	return this.id;
}

/**
 * 返回登陆者的名称
 * @return {}
 */
Login.prototype.getName = function() {
	return this.loginName;
}

/**
 * 返回登陆者的机构ID
 * @return {}
 */
Login.prototype.getOrgid = function() {
	return this.orgid;
}

/**
注销，注销成功后调用onfinish
*/
//function logoutasync(onfinish) {
//	QueryObj.create("js/login.do", 'action=logout', function(queryobj, userdata) {
//		        if (userdata) {
//			        CallBackFunc.doCallBack(userdata, queryobj);
//		        }
//		        stopPingServer();
//	        }, onfinish);
//}

UserProperties.getInstance = function(onfinish) {
	QueryObj.create(path, null, downloadUserProperties, onfinish);
}

/*用户的属性设置，模板相关设置*/
function UserProperties() {
	this.ppt = new StringMap(null, "\r\n");
}

/*返回某一系列的模板名数组 template.dim  template.dim.行业代码 template.calcparams template.groupconditions*/
UserProperties.prototype.getTemplate = function(tplnm) {
}

/*当nm不存在时，就是添加该项，如果存在就覆盖,
  是否直接写到数据库中，最好异步的写，如果连续的调用set方法，最好
  能只请求一次数据库，用timmer即可实现*/
UserProperties.prototype.set = function(nm, value) {
	this.ppt.setValue(nm, value);
	// 异步设置，
	var map = new Map();
	map.put("obj", "saveuserppt");
	map.put("name", nm);
	map.put("value", value);
	QueryObj.create("js/queryserver.do", map, function() {
	        }, null, true);
}

function getUserPpt(query, onfinish) {
	query.checkResult();
	var dtl = query.getDetail();
	onfinish(dtl);
}

UserProperties.prototype.get = function(nm, onfinish) {
	var v = this.ppt.getValue(nm);
	if (!v) {
		var map = new Map();
		map.put("obj", "getuserppt");
		map.put("name", nm);
		if (onfinish) {
			QueryObj.create("js/queryserver.do", map, getUserPpt, onfinish);
		}
		else {
			var v = queryObj("js/queryserver.do", map);
			this.ppt.setValue(nm, v);
		}
	}
	else {
		if (onfinish) {
			onfinish(v);
		}
	}
	return v;
}

UserProperties.prototype.remove = function(nm) {
	this.ppt.removeValue(nm);
}

/*从xml文件中读入模板相关属性 
 *
*/
UserProperties.prototype.importFromXml = function(xml) {
}

UserProperties.prototype.loadFromString = function(content) {
	var mp = new StringMap(content, "\r\n");
	this.ppt.putMap(mp);
}

/*服务器对象*/
function Server(wnd) {
	if (!wnd)
		wnd = window;
	this.wnd = wnd;
	this.doc = wnd.document;
	// 保存所有主题集对象
	this.tasks = new OrderMap();

	// 保存所有主题域对象
	this.grps = new OrderMap();

	this.binitgrps = false;

	this.ppt = new StringMap(); // 设置一些全局的属性

	this.login = null;
}

// 初始化Server对象，在第一次登录的时候调用该函数
Server.prototype.initServerObj = function(onfinish) {
	var self = this;
	QueryObj.create("js/login.do", 'action=getlogininfo', function(query) {
		        try {
			        query.checkResult();
			        self.login = new Login(query.getDetail());
			        onfinish(self);
		        }
		        catch (e) {
			        // TODO 出异常最好给予提示，不然根本不知道有错误发生.
			        alert(e.description);
			        // 如果获取用户登录出现异常，则注销，重新登录系统。
			        logoutasync(function() {
				                // fixed:系统异常时?id=xx&pw=xx时死循环现象
				                var loc = top.window.location;
				                loc.href = loc.href.replace(loc.search, "");
			                });
			        throw e;
		        }
	        });
}

Server.prototype.setProperty = function(k, v) {
	this.ppt.setValue(k, v);
}

Server.prototype.getProperty = function(k) {
	return this.ppt.getValue(k);
}

Server.prototype.getConf = function() {
	if (!this.conf) {
		this.conf = new ServerConf();
	}
	return this.conf;
}

// 返回地图的names，是一个数组
Server.prototype.getMapNames = function() {
	if (this.mapnames)
		return this.mapnames;
	var map = new Map();
	map.put("action", "get");
	map.put("obj", "mapnames");
	var json = queryObj("js/reqmgr.do", map);
	if (!json)
		return;
	this.mapnames = JSON.parse(json).maps;
	return this.mapnames;
}
/**
 * 根据地图ID获取地图民称
 */
Server.prototype.getMapCaptionById = function(dituname) {
	var ditus = this.getMapNames();
	for ( var i = 0, l = ditus.length; i < l; i++) {
		if (dituname == ditus[i].name) {
			return ditus[i].caption;
		}
	}
	return dituname;
}
//返回地图xml
Server.prototype.getMapTree = function(func) {
	var map = new Map();
	map.put("action", "maptree");
	 QueryObj.create("ditu.do", map,function(q){
		 try {
				q.checkResult();
				func(q.getDetail());
			} catch (e) {
				showError(e);
				return;
			}
		
	});
}

/*
 * 清空缓存的地图名称列表。当添加、删除、编辑地图之后会清空这个缓存，迫使后面重新从服务器获取地图列表
 */
Server.prototype.clearMapNames = function() {
	if (this.mapnames) this.mapnames=null;
}

/*返回系统中的datasource个数,包含系统默认的*/
Server.prototype.getDatasouceCount = function() {
	this._initDbServer();
	return this.dbsvrMap.size();
}

Server.prototype.getDatasouceName = function(i) {
	this._initDbServer();
	return this.dbsvrMap.getKey(i);
}

/*也可以是dbserver的name*/
Server.prototype.getDbServer = function(nm) {
	this._initDbServer();
	var dbserver = this.dbsvrMap.get(nm);
	if (!dbserver) {
		if (typeof(DbServer) == 'undefined')
			sys.lib.include('ebi/js/sql.js');
		dbserver = new DbServer(nm);
		this.dbsvrMap.put(nm, dbserver);
	}
	return dbserver;
}

/**
 * 读取系统连接池列表，默认的连接池为空串
 */
Server.prototype._initDbServer = function() {
	if (!this.dbsvrMap) {
		this.dbsvrMap = new OrderMap();
		var str = queryObj("js/queryserver.do", "obj=datasource");
		var arr = str.split("\r\n");
		for (var i = 0; i < arr.length; i++) {
			this.dbsvrMap.put(arr[i], null);
		}
	}
}

Server.prototype.getVfs = function() {
	if (this.vfs) {
		return this.vfs;
	}
	if (typeof(Vfs) == 'undefined')
		sys.lib.include('ebi/js/vfs.js');
	this.vfs = new Vfs();
	return this.vfs;
}

Server.prototype.dispose = function() {

}

Server.prototype.getTheme = function() {
	if (!this.theme) {
		this.theme = queryObj("js/queryserver.do", "obj=theme");
	}
	return this.theme;
}

/*返回全局的自定义函数CustomFuncs对象*/
Server.prototype.getCustomFuncs = function(refresh) {
	var b = typeof(refresh) == 'boolean' ? refresh : false;
	if (!this.customfuncs || b) {
		var content = queryObj("js/queryserver.do", "obj=downloadfuncs&type=server");
		if (typeof(CustomFuncs) == "undefined")
			sys.lib.include("ebi/js/customfuncs.js");
		var f = new CustomFuncs();
		f.loadFromStr(content, false);
		this.customfuncs = f;
	}
	return this.customfuncs;
}

Server.prototype.getLogin = function(reget/*是否重新获取login对象*/) {
	if (!this.login || reget) {
		this.login = new Login();
	}
	return this.login;
}

Server.prototype.getServerTime = function(format){
	var datetime = queryObj("js/login.do", 'action=getservertime');
	if(format == "") return datetime;
	else return date2String(new Date(Date.parse(datetime)), format);
}

/*获取window对象*/
Server.prototype.getWindow = function() {
	return this.wnd;
}

/*获取document对象*/
Server.prototype.getDocument = function() {
	return this.doc;
}

// 主题集信息是和主题域信息一起获得的
Server.prototype._loadTasksInfo = function(dom, grpobj) {
	var nds = dom.childNodes;
	for (var i = 0, len = nds.length; i < len; i++) {
		var nd = nds[i];
		var uid = nd.getAttribute("uid");
		var taskid = nd.getAttribute("name");
		var caption = nd.getAttribute("caption");
		var owner = nd.getAttribute("owner");
		var task = new Task(uid, taskid, caption, grpobj.getId());
		this.tasks.put(taskid, task);
		grpobj.tasks.put(taskid, task);
	}
}

Server.prototype._loadGrpsInfo = function(dom) {
	if (dom.documentElement)
		dom = dom.documentElement;
	var nds = dom.childNodes;
	for (var i = 0, len = nds.length; i < len; i++) {
		var nd = nds[i];
		var uid = nd.getAttribute("uid");
		var name = nd.getAttribute("name");
		var caption = nd.getAttribute("caption");
		var owner = nd.getAttribute("owner");
		var datasource = nd.getAttribute("datasource");
		var grp = new TaskGroup(this, name, caption, owner, datasource);
		grp.uid = uid;
		this.grps.put(name, grp);
		this._loadTasksInfo(nd, grp);
	}
}

Server.prototype.initGrps = function(onfinish) {
	if (this.binitgrps) {
		if (onfinish)
			onfinish();// 如果已经初始化了grps，那么直接调用onfinish
		return;
	}
	var url = "js/listresource.do?rid=EBI$0$server&level=2&types=EBI$3$,EBI$7$"; // 列出主题域和主题集的信息
	if (onfinish) {
		QueryObj.create(url, null, function(xmlReq, self) {
			        xmlReq.checkResult();
			        self._loadGrpsInfo(xmlReq.getResponseXML());
			        onfinish();
		        }, this, true);
	}
	else {
		var dom = queryXml(url);
		this._loadGrpsInfo(dom);
	}
	this.binitgrps = true;
}

Server.prototype.initTasks = Server.prototype.initGrps;// 兼容

Server.prototype.getContextPath = function() {
	return this.contextPath;
}

/*添加删除主题集以后，重新从服务器上请求某个组的主题集*/
Server.prototype.reloadGroups = function(onfinish) {
	this.grps.clear();
	this.tasks.clear();
	this.binitgrps = false;
	this.initGrps(onfinish);
}

Server.prototype.getTask = function(taskid, throwIfNotExists) {
	this.initTasks();
	var tasks = this.tasks;
	var tsk = tasks.get(taskid);
	if (!tsk && throwIfNotExists) {
		//throwError("主题集不存在：" + taskid);
		throwError(I18N.getString("ebi.user.main.objsatroot.js.str2","主题集不存在：{0}",[taskid]));
	}
	return tsk;
}

/*返回主题域的个数*/
Server.prototype.getTaskGroupCount = function() {
	this.initGrps();
	return this.grps.size();
}

/*返回第i个主题域，如果i为字符串，则返回id为i的主题域*/
Server.prototype.getTaskGroup = function(i) {
	this.initGrps();
	if (typeof(i) == "string") {
		return this.grps.get(i);
	}
	else if (typeof(i) == "number") {
		return this.grps.getByIndex(i);
	}
	else {
		//alert("参数错误");
		alert(I18N.getString("ebi.user.main.objsatroot.js.str3","参数错误"));
	}
}

/*服务器属性对象,和服务端的ServerConf.java对应*/
function ServerConf() {
	this.ischange = true;
}

ServerConf.prototype.queryServerConf = function() {
	if (!this.serverconf || this.ischange) {
		this.serverconf = queryObj("js/queryserver.do", "obj=serverconf");
		this._load(this.serverconf);
		this.ischange = false;
	}
}

ServerConf.prototype.modifyyServerConf = function() {
	queryObj("js/queryserver.do", "content=" + this.ppt.toString() + "&obj=modifyserverconf");
}

ServerConf.prototype.changed = function() {
	this.ischange = true;
}

ServerConf.prototype._load = function(str) {
	this.ppt = new StringMap(str, "\r\n");
}

ServerConf.PLUGIN_PRINTER = 'printer';// 打印插件
ServerConf.PLUGIN_DESIGNER = 'sanrptdsn';// 设置插件
ServerConf.PLUGIN_VIEW = 'sanreportview';// 显示插件
ServerConf.prototype.getPluginCodeBase = function(plugin) {
	return this.getProperty(plugin);
}

ServerConf.REPORT_VIEW_TYPE = "ReportViewType";
ServerConf.OLAP_ROWS = "olaprows";
ServerConf.QBE_ROWS = "qberows";
ServerConf.TEMPLETE_ROWS = "templeterows";
ServerConf.prototype.getReportViewType = function() {
	return this.getProperty(ServerConf.REPORT_VIEW_TYPE);
}

ServerConf.prototype.setReportViewType = function(s) {
	this.setProperty(ServerConf.REPORT_VIEW_TYPE, s);
}

ServerConf.prototype.getPageSize = function(rpttype) {
	switch (rpttype) {
		case RT_OLAP :
			return this.getOlapRows();
		case RT_FORMATCAL :
			return this.getTempleteRows();
		case RT_QBE :
			return this.getQbeRows();
	}
}

/*判断当前的服务器是否可以使用这些模块，如果不能使用，就不显示*/
ServerConf.MODULE_QBE = "BI_MODULE_QBE";
ServerConf.MODULE_OLAP_DLG = "BI_MODULE_DLGOLAP";
ServerConf.MODULE_OLAP_EASY = "BI_MODULE_EASYOLAP";
ServerConf.MODULE_REPORT = "BI_MODULE_REPORT";
ServerConf.MODULE_SCHEDULE = "BI_MODULE_SCHEDULE";
ServerConf.MODULE_CHART = "BI_MODULE_CHART";

ServerConf.prototype.checkModules = function(moduleName) {
	return this.getProperty(moduleName) ? false : true;
}

ServerConf.prototype.getQbeRows = function() {
	return this.getProperty(ServerConf.QBE_ROWS);
}

ServerConf.prototype.setQbeRows = function(s) {
	this.setProperty(ServerConf.QBE_ROWS, s);
}

ServerConf.prototype.getOlapRows = function() {
	return this.getProperty(ServerConf.OLAP_ROWS);
}

ServerConf.prototype.setOlapRows = function(s) {
	this.setProperty(ServerConf.OLAP_ROWS, s);
}

ServerConf.prototype.getTempleteRows = function() {
	return this.getProperty(ServerConf.TEMPLETE_ROWS);
}

ServerConf.prototype.setTempleteRows = function(s) {
	this.setProperty(ServerConf.TEMPLETE_ROWS, s);
}

/*
 * reportviewtype 报表显示方式0为DHTML,1为plugin
*/
ServerConf.prototype.getProperty = function(key, def) {
	this.queryServerConf();
	return this.ppt.getValue(key, def);
}

ServerConf.prototype.setProperty = function(key, value) {
	this.ppt.setValue(key, value);
	this.modifyyServerConf();
}

/*主题域对象*/
TaskGroup = function(server, id, caption, owner, datasource) {
	this.server = server;
	this.id = id;
	this.caption = caption;
	this.owner = owner;
	this.datasource = datasource;
	this._className = "TaskGroup";
	this.tasks = new OrderMap();
}

TaskGroup.prototype.getResourceId = function() {
	return "EBI$"+3 + '$' + this.id;
}

/*返回主题域id，即名称*/
TaskGroup.prototype.getId = function() {
	return this.id;
}

/*返回主题域的标题*/
TaskGroup.prototype.getCaption = function() {
	return this.caption;
}

/*返回该主题域的创建者*/
TaskGroup.prototype.getOwner = function() {
	return this.owner;
}
/*返回主题域的数据库连接池名称*/
TaskGroup.prototype.getDataSource = function() {
	return this.datasource;
}
/*返回此组内的此登陆者可以访问的主题集对象的数目*/
TaskGroup.prototype.getTaskCount = function() {
	this.server.initTasks();
	var map = this.tasks;
	return map ? map.size() : 0;
}

/*返回此组内的此登陆者可以访问的第i个主题集对象*/
TaskGroup.prototype.getTask = function(i) {
	this.server.initTasks();
	var map = this.tasks;
	return map.getByIndex(i);
}

/*主题集对象，构造参数taskid，表示此主题集的serverid，此函数会利用xmlhttp对象
  从服务器上请求下主题集的文件然后读取文件中的配置*/
function Task(uid, taskid, caption, ownergroupid) {
	this.uid = uid;
	this.taskid = taskid;
	this.ownergroupid = ownergroupid;// 主题集所数的主题域的id
	this.caption = caption;
	this.map = new OrderMap();// 保存报表对象
	this.xxb = null; // 就保存xxb对象
	//this.bbhIdTitle = "主代码";// 返回报表户id用户自定义称谓
	this.bbhIdTitle = I18N.getString("ebi.user.main.objsatroot.js.str4","主代码");// 返回报表户id用户自定义称谓
	//this.bbhNameTitle = "主名称";
	this.bbhNameTitle = I18N.getString("ebi.user.main.objsatroot.js.str5","主名称");
	this._isinit = false;
	this._isinittaskdatainfo = false;
	this.taskDataInfo = null;
	this._className = "Task";
}
// 释放资源
Task.prototype.dispose = function() {
	if (this.map)
		this.map = null;
	if (this.taskDataInfo) {
		this.taskDataInfo.owner = null;
		this.taskDataInfo = null;
	}
}

/*返回该主题集所属的主题域*/
Task.prototype.getOwnerGroupId = function() {
	return this.ownergroupid;
}

/*要采用异步方式，下载主题表，如果onfinish不为空，就采用异步的方式下载*/
Task.prototype._init = function(onfinish) {
	if (this._isinit) {
		if (onfinish)
			onfinish(this);
		return;
	}

	if (onfinish) {
		var cbs = this._init_onfinish_listeners;
		if (!cbs) {
			cbs = new CallBackFuncs();
			this._init_onfinish_listeners = cbs;
		}
		cbs.add(onfinish);
		if (this._initing) {
			return;
		}
		this._initing = true;
		QueryObj.create("js/querytask.do", "taskid=" + this.taskid + "&obj=facttable", this._init_onfinish, this, true);
	}
	else {
		var dmtxt = queryObj("js/querytask.do", "taskid=" + this.taskid + "&obj=facttable");
		tsk = this;
		eval(dmtxt);
		this._isinit = true;
	}
}

Task.prototype._init_onfinish = function(query, self) {
	self._initing = false;
	self._isinit = true;
	query.checkResult();
	tsk = self;
	eval(query.getDetail());
	var cbs = self._init_onfinish_listeners;
	if (cbs) {
		cbs.invoke(self);
		cbs.clear();
	};
}

Task.prototype.ensureLoad = Task.prototype._init;

Task.prototype.reload = function() {
	this._isinit = false;
	this.map.clear();
}

Task.prototype._initTaskDataInfo = function() {
	if (this._isinittaskdatainfo)
		return;
	this.taskDataInfo = new TaskDataInfo();
	this.taskDataInfo.owner = this;
	this._isinittaskdatainfo = true;
}

Task.prototype.getTaskDataInfo = function() {
	this._initTaskDataInfo();
	return this.taskDataInfo;
}

/* 返回数据对应的名称，用';'分隔 */
Task.prototype.getDatainfoPlanNames = function() {
	if (!this.dipinited) {
		this.dipnames = queryObj("js/querytask.do", "taskid=" + this.taskid + "&obj=datainfoplan");
		this.dipinited = true;
	}
	return this.dipnames;
}

// 返回报表户id用户自定义称谓
Task.prototype.getBbhIdTitle = function() {
	//return '主代码';
	return I18N.getString("ebi.user.main.objsatroot.js.str4","主代码");
}

// 返回报表户名称用户自定义称谓
Task.prototype.getBbhNameTitle = function() {
	//return '主名称';
	return I18N.getString("ebi.user.main.objsatroot.js.str5","主名称");
}

/*获得此主题集的id*/
Task.prototype.getTaskId = function() {
	return this.taskid;
}

Task.prototype.getResourceId = function() {
	return "EBI$"+ 7 + '$' + this.taskid;
}

Task.prototype.getId = Task.prototype.getTaskId;

/*返回主题集对应的serverid*/
Task.prototype.getServerId = Task.prototype.getTaskId;

/*获得此主题集的详细名称，返回字符串*/
Task.prototype.getCaption = function() {
	return this.caption;
}

/*返回此主题集中数据报表的数目, xxb, basic, planar*/
Task.prototype.getDataReportCount = function() {
	this._init();
	return this.map.size();
}

/*如果i为整形表示获得第i个数据报表的对象
  如果i为字符串则表示获得表名为i的报表对象，大小写不敏感，
  ifnot..参数只对当i是字符串时有效，返回Report对象*/
Task.prototype.getDataReport = function(i, ifnotExistsThenThrow) {
	this._init();
	if (isNumber(i)) {
		return this.map.getByIndex(i);
	}
	else if (isString(i)) {
		/**
		 * BI-3484 由于在2.2M2中修改了主题表的资源名，在主题表后面带上了.fact或者.vfact，常量定义在AnaConst.java中
         * 
         * BI-5595 这里增加一个主题就要修改一次，比较繁琐，不过这样判断完善一些。
		 */
		var s = i.toUpperCase();
		if (s.endsWith(".FACT") || s.endsWith(".VFACT") || s.endsWith(".CUBE")|| s.endsWith(".LINK")) {
			var idx = s.lastIndexOf(".");
			if (idx != -1) {
				s = s.substring(0, idx);
			}
		}
		var rpt = this.map.get(s);
		if ((rpt == null) && (arguments.length > 1) && (ifnotExistsThenThrow)) {
			//throwError("所获取的" + i + "表不存在");
			throwError(I18N.getString("ebi.user.main.objsatroot.js.str6","所获取的{0}表不存在",[i]));
		}
		return rpt;
	}
	else {
		return null;
	}
}

/*获得信息表对象，返回Report对象*/
Task.prototype.getXxb = function() {
	this._init();
	return this.xxb;
}

/*clnm为Xxb->A2或者B0->B6之类的串，此函数返回对应的cell对象。
  XXB.ZZFI_B之类的对应数据库的字段，onfinish
  如果cell不存在则返回null*/
Task.prototype.getCell = function(clnm, onfinish, cellnameFirst) {
	if (!clnm || clnm.length == 0) {
		if (onfinish) {
			onfinish(null)
		};
		return null;
	}
	if (onfinish) {
		var self = this;
		this.ensureLoad(function(tsk) {
			        self._getcell(tsk, clnm, onfinish, cellnameFirst);
		        });
	}
	else {
		return this._getcell(this, clnm, onfinish, cellnameFirst);
	}
}

Task.prototype._getcell = function(tsk, clnm, onfinish, cellnameFirst) {
	if (!onfinish)
		tsk._init();
	var rptnm = '';
	var i = clnm.indexOf(".");
	if (i > -1) {// xxb.A2
		rptnm = clnm.substring(0, i);
		clnm = clnm.substring(i + 1);
	}
	else {
		i = clnm.indexOf("->");
		if (i > -1) {
			rptnm = clnm.substring(0, i);
			clnm = clnm.substring(i + 2);
		}
		else {
			if (onfinish)
				onfinish(null);
			return null;
		}
	}
	var rpt = tsk.map.get(rptnm.toUpperCase());
	if (onfinish) {
		if (rpt) {
			/**
			 * getSelfField参数只接收当前报表的别名、字段名和表元名，不带的报表的，如 USERID，而不是xxb.userid
			 */
			rpt.ensureLoad(function(r) {
				        onfinish(r.getSelfField(clnm, false, cellnameFirst));
			        });
		}
		else {
			onfinish(null);
		}
	}
	else {
		var cell = rpt != null ? rpt.getSelfField(clnm, false, cellnameFirst) : null;
		return cell;
	}
}

/**
 * 根据一个以逗号分割的表元名的串，返回一个Cell数组对象，如果clnms中的表员没有带表名，则用rptname替代
 * 
 * yk 20090804 BI-2264 拾取表元，没有显示指标字段名，而是显示指标代号
 * 原因是E43的字段名是N_BBQ，但还有个表元E45它的字段名是E43，这导致拾取E43时根据getCells获得的表元对象
 * 是E45，故出现问题，此处增加参数cellnameFirst，用于判断是否以表元名为优先去获得表元，默认情况下还是以
 * 别名和字段名为优先。
 * 
 */
Task.prototype.getCells = function(rptname, clnms, ifnotExistsThenThrow, cellnameFirst) {
	var cellArray = new Array();
	if (clnms == null || clnms.length == 0)
		return cellArray;
	this._init();
	var clnames = clnms.split(",");
	var cell = null;
	var clname;
	var defrptobj;
	for (var i = 0, len = clnames.length; i < len; i++) {
		clname = clnames[i];
		if (clname != null && clname.length > 0) {
			if (clname.indexOf(".") == -1 && clname.indexOf("->") == -1) {
				if (!defrptobj)
					defrptobj = this.getDataReport(rptname);
				cell = defrptobj.getSelfField(clname, ifnotExistsThenThrow, cellnameFirst);
			}
			else {
				cell = this.getCell(clname, null, cellnameFirst);
				if ((cell == null) && ifnotExistsThenThrow) {
					//throwError(clname + "表元不存在");
					throwError(I18N.getString("ebi.user.main.objsatroot.js.str7","{0}表元不存在",[clname]));
				}
			}
		}

		cellArray.push(cell);
	}

	return cellArray;
}

/*返回该主题集的所有代码组维的名字的数组DimsMgr返回该对象*/
Task.prototype.getTaskDims = function(onfinish) {
	if (onfinish) {
		if (this.dimsMgr) {
			onfinish(this.dimsMgr);
		}
		else {
			var self = this;
			QueryObj.create("js/querytask.do?taskid=" + this.taskid + "&obj=dims", null, function(q) {
				        q.checkResult();
				        self.dimsMgr = new DimsMgr(q.getDetail());
				        onfinish(self.dimsMgr);
			        });
		}
	}
	else {
		if (!this.dimsMgr) {
			var str = queryObj("js/querytask.do?taskid=" + this.taskid + "&obj=dims");
			if (str) {
				this.dimsMgr = new DimsMgr(str);
			}
		}
		return this.dimsMgr;
	}
}

/*管理某一主题域的所有维var dimobj = {"":[],"":[]}，维的定义，数据级次维($DIM_PID) 企业id维($DIM_MID)*/
function DimsMgr(str) {
	this.dimPidName;
	this.dimMidName;
	eval(str);
	this.objMap = new OrderMap();// 用来代码名称和其对应的属性数组的
	for (var key in dimobj) {
		if (key == '$TASKGROUPPPT') { // 数据级次维做特殊处理，第一个值存的是数据级次维的名称，第二个值存的是企业ID维
			this.dimPidName = dimobj[key][0];
			this.dimMidName = dimobj[key][1];
		}
		else {
			this.objMap.put(key, new DimDefine(key, dimobj[key]));
		}
	}
}

/*name:行业代码；返回该行业的属性数组[{name:'行业大类',fld:'',desc:''},{...},...] fld字段名，desc字段显示名 */
DimsMgr.prototype.getDims = function(name) {
	// 当是($DIM_PID) 和 ($DIM_MID)时，就返回对应的 数据级次维名和企业维名
	if (name == '$DIM_PID') {
		name = this.dimPidName;
	}
	else if (name == '$DIM_MID') {
		name = this.dimMidName;
	}
	return this.objMap.get(name);
}

/*顺序返回每个维的name 行业代码*/
DimsMgr.prototype.getDimName = function(i) {
	return this.objMap.getKey(i);
}

/*返回有多少个维*/
DimsMgr.prototype.getCount = function() {
	return this.objMap.size();
}

/*如果是层级维，obj.type drill|ppt|lsc|def(层次维默认的属性)， 直接在服务器端生成好了，传给客户端*/
function DimDefine(name, arr) {
	this.name = name;
	this.pptlist = new OrderMap(); // 保存的是属性列表
	this.level = -1;
	if (arr.length > 0) {
		this.level = arr[0].level;
	}
	for (var i = 1; i < arr.length; i++) {
		var obj = arr[i];
		this.pptlist.put(obj.name.toUpperCase() + '', new DimPropertyItem(obj.name + "", obj.desc, obj.type));
	}
}

DimDefine.prototype.getTotalLevel = function() {
	return this.level;
}

// 返回一个显示维属性，分段名称，以及钻取级次的列表 showlevel表示是否把层级维转换为第一级，第二级，等等的
// 如果是个单级维，没有属性，那么就不显示第一级什么的
DimDefine.prototype.getShowPptList = function() {
	return this.pptlist;
}

/*维定义 属性对象 type =ppt(属性)|lsc(分段名称)|drill(钻取路径) */
function DimPropertyItem(nm, desc, type) {
	this.name = nm;
	this.desc = desc;
	this.type = type;
}

/*该函数用于给服务器端生成JS使用*/
FactTable.create = function(task, nm, type, caption, alias, grp, bbhid, bbhname, isaddname, isaddid) {
	var rpt = new FactTable(task, nm, type, caption, alias, grp, bbhid, bbhname, isaddname, isaddid);
	// 如果是信息表，给其单独赋值
	if (type == 0)
		task.xxb = rpt;
	task.map.put(nm.toUpperCase(), rpt);
}

/*报表对象*/
function FactTable(task, nm, type, caption, alias, grp, bbhid, bbhname, isaddname, isaddid) {
	this.task = task;
	this.nm = nm;
	this.type = type;
	this.caption = caption;
	this.alias = alias;
	this.grp = grp;
	this.bbhid = bbhid;
	this.bbhname = bbhname;
	this.isaddname = isaddname;
	this.isaddid = isaddid;
	this.zbsArray = new Array(); // 保存CELL对象,按顺序
	this.zbsMap = new Map();// 以别名、字段名、表元名为KEY，保存CELL对象
	this.dimarr = []; // 只记录维字段名，取factField时，在getField取
}

FactTable.prototype.getName = function() {
	return this.nm;
}

FactTable.prototype.getCaption = function() {
	return this.caption;
}

FactTable.prototype.getType = function() {
	return this.type;
}

/*报表简称*/
FactTable.prototype.getAlias = function() {
	return this.alias;
}

/*返回该表所属组名*/
FactTable.prototype.getGroup = function() {
	return this.grp;
}

FactTable.prototype.getTask = function() {
	return this.task;
}

FactTable.prototype.isAddName = function() {
	return this.isaddname;
}

FactTable.prototype.isAddId = function() {
	return this.isaddid;
}

// 是否是虚拟主题
FactTable.prototype.isVirtualFact = function() {
	return parseInt(this.type, 10) == 2;
}

/**
 * 是否是立方体主题
 * @return {}
 */
FactTable.prototype.isCubeFact = function(){
    return parseInt(this.type, 10) == 4;
}

/*获取该主题表的报表户id名称*/
FactTable.prototype.getBbhIdTitle = function() {
	return this.bbhid;
}

/*获取该主题表的单位名称*/
FactTable.prototype.getBbhNameTitle = function() {
	return this.bbhname;
}

/*根据别名、字段名或名称获得表元对象，所传递的clnm可能为如下形式xxb->b1、xxb->b2、A1
  如果传递的表员名中带表名，但表明不是此表的，则能从task对象获得对应的cell*/
FactTable.prototype.getCell = function(alias_dbfield_name, ifNotExistsThenThrow) {
	/**
	 * 此处必须以表元名为优先，因为拾取的时候传递的是表元名
	 */
	var clnm = alias_dbfield_name;
	var i = clnm.indexOf('.');
	var j = clnm.indexOf('->');
	var rpt = this;
	if (i > 0) {
		rpt = this.task.getDataReport(clnm.substring(0, i), false);
		if (rpt == null) {
			return null;
		}
		clnm = clnm.substring(i + 1);
	}

	if (j > 0) {
		rpt = this.task.getDataReport(clnm.substring(0, j), false);
		if (rpt == null) {
			return null;
		}
		clnm = clnm.substring(j + 2);
	}

	/**
	 * 此时的rpt可能不等于this
	 * 这里不能直接调用rpt.zbsMap，因为只有rpt初始化了zbsMap才有值
	 * 在用虚拟主题做easyolap时，一进入easyolap界面就选择数据期对话框，此时系统要分析显示哪个主题的数据期信息，这时会出现这个问题。
	 */
	var cl = rpt.getField(clnm);

	if (cl == null && ifNotExistsThenThrow) {
		//throwError(alias_dbfield_name + "表元不存在。");
		throwError(I18N.getString("ebi.user.main.objsatroot.js.str7","{0}表元不存在",[alias_dbfield_name]));
	}
	return cl;
}

/**
 * 根据字段名，获取表元对象
 * 此函数改为直接调用getCell，没必要只根据字段名获取指标
 */
FactTable.prototype.getFactFieldByFiledName = FactTable.prototype.getCell;

// 解析该主题表的XML文件，一次遍历，能够记录所有字段，还要能把指标和维遍历出来，分组显示
/*
<?xml version="1.0" encoding="gb2312"?>
<factTable>
  <options>
    <option name="" value=""/>
  </options>
  <zbs>
    <grp caption="指标">
      <zb name="MSXSE" datatype="N" caption="免税销售额" exp="sbxx_hz.MSXSE"/>
      <zb name="CKMSXSE" datatype="N" caption="出口免税货物销售额" exp="sbxx_hz.CKMSXSE"/>
      <zb name="CKMDTXSE" datatype="N" caption="出口免抵退税货物销售额" exp="sbxx_hz.CKMDTXSE"/>
      <zb name="QMLDS" datatype="N" caption="期末留抵数" exp="sbxx_hz.QMLDS"/>
    </grp>
    <zb name="nsrdzdah" datatype="C" caption="纳税人电子档案号" exp="sbxx_hz.nsrdzdah" isdim=true dim=""/>
    <zb name="djzclx_dm" datatype="C" caption="登记注册类型代码" exp="sbxx_hz.djzclx_dm" isdim=true dim="登记注册类型"/>
    <zb name="hy_dm" datatype="C" caption="行业代码" exp="sbxx_hz.hy_dm" isdim=true dim="行业"/>
    <zb name="JDXZ_DM" datatype="C" caption="街道乡镇代码" exp="sbxx_hz.JDXZ_DM" isdim=true dbfield=""/>
  </zbs>
  <filters>
    <filter name="abc" caption="免税销售额大于1000" exp="sbxx_hz.MSXSE>1000"/>
  </filters>
</factTable> 
解析zbs下的
*/
FactTable.prototype._parseFactTableXml = function(xml) {
	if (!xml)
		return;
	var root = xml.documentElement;
	if (!root)
		return;
	var nodes = root.childNodes;
	for (var i = 0, len = nodes.length; i < len; i++) {
		var node = nodes[i];
		var tagname = node.tagName;
		if (tagname == 'options') {
		}
		else if (tagname == 'zbs' || tagname == 'fields') { // 读取指标
			this._parseZbsXml(node, '');
		}
		else if (tagname == 'filters') {// 读取过滤条件
			// 过滤条件不在从客户端对象读取
		}
	}
}

// 解析zbs XML
FactTable.prototype._parseZbsXml = function(zbsnode, group) {
	var grp = group;
	var childnode = zbsnode.childNodes;
	for (var i = 0, len = childnode.length; i < len; i++) {
		var node = childnode[i];
		if (node.nodeType != 1)
			continue;
		var tagname = node.tagName;
		//兼容BI2.3新的虚拟主题或者物理主题格式
		if (tagname == 'zb' || tagname == 'field') {
			var zb = this._parseNodeXml2FactField(node, grp);
			this.zbsArray.push(zb);
			this.zbsMap.put(zb.getName().toUpperCase(), zb);
			if (zb.getFieldName()) { // 如果改指标有表元名，那么就保存在fldnmMap中
				this.zbsMap.put(zb.getFieldName().toUpperCase(), zb);
			}
			if (zb.getAlias()) {// 别名优先级最高，所以放到最后
				this.zbsMap.put(zb.getAlias().toUpperCase(), zb);
			}
			if (zb.isDimension()) {// 如果是维，就加在维iterator里面
				this.dimarr.push(zb.getName());
			}
			else {
				if (!this.isVirtualFact()) {
					//zb.group = '指标';
					zb.group = I18N.getString("ebi.user.main.objsatroot.js.str8","指标");
				}
			}
		}
		if (node.childNodes.length > 0) {
			if (grp) {
				group = grp + "\\" + node.getAttribute("caption");
			}
			else {
				group = node.getAttribute("caption");
			}
			this._parseZbsXml(node, group);
		}
	}
}

// <zb name="JDXZ_DM" datatype="C" caption="街道乡镇代码" exp="sbxx_hz.JDXZ_DM" isdim=true dbfield=""/>
FactTable.prototype._parseNodeXml2FactField = function(node, group) {
	var name = node.getAttribute("name");
	var alias = node.getAttribute("alias");
	var datatype = node.getAttribute("datatype");
	var caption = node.getAttribute("caption");
	var exp = node.getAttribute("exp");
	/**
	 * BI-5084 由于在BI3.1中逻辑主题的格式修改了，导致没法直接读出exp，
	 * <field>
	 * <exp>HR_B1.BBQ_</exp>
	 * </field> 
	 */
	if(!exp){
		var nn = getFirstChild(node);
		if(nn){
			exp = isie ? nn.text : nn.textContent;
		}
	}
	var isdim = node.getAttribute("isdim");
	var dbfield = node.getAttribute("dbfield");
	var dim = node.getAttribute("dim");
	var options = node.getAttribute("options");
	return new FactField(name, alias, caption, dbfield, exp, group, datatype, parseBool(isdim, false), dim, options, this.nm);
}

FactTable.prototype._initFactTable = function(onfinish) {
	if (this._initZbs) {
		if (onfinish)
			onfinish(this);
		return;
	}
	var params = new Map();// isLogicFact
	params.put("taskid", this.task.getTaskId());
	params.put("obj", "facttableasxml");
	params.put("rpt", this.nm);
	if (onfinish) {
		/**
		 * BIDEV-522 避免客户端多次请求，每次都往服务器发送请求，记录每次请求的onfinish，当请求完成时，逐一执行。
		 */
		var cbs = this._init_onfinish_listeners;
		if (!cbs) {
			cbs = new CallBackFuncs();
			this._init_onfinish_listeners = cbs;
		}
		cbs.add(onfinish);

		if (this._initing) {
			return;
		}
		this._initing = true;

		new QueryObj('js/querytask.do', params, this._parseQuery, this, false);
	}
	else {
		var q = new QueryObj('js/querytask.do', params, null, null, false);
		this._parseQuery(q, this);
	}
}

FactTable.prototype._parseQuery = function(query, ref) {
	ref._initing = false;
	var xml = query.getResponseXML();
	ref._parseFactTableXml(xml);
	ref._initZbs = true;

	var cbs = ref._init_onfinish_listeners;
	if (cbs) {
		cbs.invoke(ref);
		cbs.clear();
	};
}

FactTable.prototype.ensureLoad = FactTable.prototype._initFactTable;

FactTable.prototype.getFieldCount = function() {
	this._initFactTable();
	/**
	 * 修改wtap时，该表了this.zbsArray的类型，以前是使用的OrderMap，现在换成数组对象了，故导致编辑报表模版时，不能提示主题表的字段信息
	 */
	return this.zbsArray.length;
}

/**
 * 获得本表的指标，参数可以是序号或者是名称
 */
FactTable.prototype.getField = function(alias_dbfield_name, throwIfNotExists) {
	this._initFactTable();
	var key = alias_dbfield_name;
	var fld;
	if (isString(key)) {
		fld = this.zbsMap.get(key.toUpperCase());
	}
	else {
		fld = this.zbsArray[key];
	}

	if (!fld && throwIfNotExists) {
		//throwError('字段' + alias_dbfield_name + "不存在!");
		throwError(I18N.getString("ebi.user.main.objsatroot.js.str9","字段{0}不存在!",[alias_dbfield_name]));
	}
	return fld;
}

/**
 * 获得本表的指标，参数可以是序号或者是名称
 * 参数clnamefirst表示是否以表元名为优先，因为拾取时是以表元名为依据的。
 */
FactTable.prototype.getSelfField = function(alias_dbfield_name, throwIfNotExists, clnamefirst) {
	if (clnamefirst) {
		return this.getSelfFactFieldByCellName(alias_dbfield_name, throwIfNotExists);
	}

	return this.getField(alias_dbfield_name, throwIfNotExists);
}

/**
 * 根据表元名（不支持别名和字段名）获取本表的表元对象，此函数一般只用在拾取时，因为拾取时鼠标点击的dom上获得的是clname，
 * 然后根据clname在获得cell对象，再决定使用表元还是别买还是字段名来引用表元。
 */
FactTable.prototype.getSelfFactFieldByCellName = function(cellname, throwIfNotExists) {
	this._initFactTable();
	var zbs = this.zbsArray;
	var zb;
	cellname = cellname.toUpperCase();
	for (var i = 0; i < zbs.length; i++) {
		zb = zbs[i];
		if (zb.getName().toUpperCase() == cellname) {
			return zb;
		}
	}

	if (throwIfNotExists) {
		//throwError(cellname + "表元不存在。");
		throwError(I18N.getString("ebi.user.main.objsatroot.js.str7","{0}表元不存在",[cellname]));
	}

	return null;
}

/*返回该表中维的个数*/
FactTable.prototype.getDimCount = function() {
	this._initFactTable();
	return this.dimarr.length;
}

/*返回该表中的第i个维*/
FactTable.prototype.getDim = function(i) {
	this._initFactTable();
	var nm = this.dimarr[i];
	return this.getField(nm);
}

FactTable.prototype.toString = function() {
	return this.caption + "(" + this.nm + ")";
}

//-----------报表中的单元格对象-----------------------------
/**
 * 报表中的单元格对象，该对象在拾取时使用，同FactField有区别，FactField是主题表中每一个事实的描述，
 * 而Cell是一个报表单元格的描述，FactField从Cell继承
 */
/**
 * add by kangx ISSUE:ESENBI-63
 * 编辑界面也存在Cell对象，表示的是编辑界面上表格表元对象。
 * 这里的cell对象仅在结果表显示拾取的时候才会使用，
 * 且入口已经统一createCellObjImpl，所以修改量小。
 * 
 * 修改该对象的名字为CellInfo
 */


createCellObjImpl = function(nm, dataType, caption, dbField, alias, rpt) {
	var cl = new CellInfo(nm, caption, dbField, alias, dataType);
	cl.rpt = rpt;
	return cl;
}

/*表元对象*/
function CellInfo(name, caption, dbfld, alias, datatype) {
	this.name = name;
	this.dbfld = dbfld;
	if (alias)
		this.alias = alias;
	this.caption = caption || alias || dbfld;
	this.datatype = datatype;
	this.rpt = null;
}

/*是否是一个带表名的表元名，如b0->a1, bsdfsdf_sdf.dfgdf, werw34-ert34sf->er34，只有ExpChecker*/
CellInfo._fullCellNamePattern = /([a-zA-Z_]\w*\-\>[a-zA-Z_]\w*)|([a-zA-Z_]\w*[.][a-zA-Z_]\w*)/g;
CellInfo.isFullCellName = function(clnm) {
	if (!clnm) {
		return false;
	}
	var r = clnm.match(CellInfo._fullCellNamePattern);
	return r && (r[0] == clnm);
}

CellInfo.prototype._className = 'CellInfo';

// 是否是固定表元
CellInfo.prototype.isFixCell = function() {
	return this.datatype == null || this.datatype.length == 0;
}

// 是否是数值表元
CellInfo.prototype.isNumCell = function() {
	return this.datatype == 'I' || this.datatype == 'N';
}

// 是否是数值表元
CellInfo.prototype.isDataCell = function() {
	return (this.datatype != null && this.datatype.length == 1);
}

/*判断表元是否是字符类型，包括字符型数据表元和固定表元*/
CellInfo.prototype.isStrCell = function() {
	return this.isStrDataCell() || (this.datatype == null);
}

/*判断表元是否是字符型数据表元*/
CellInfo.prototype.isStrDataCell = function() {
	return this.datatype == "C";
}

CellInfo.prototype.getDataType = function() {
	return this.datatype;
}

CellInfo.prototype.getName = function() {
	return this.name;
}

//获取单元格的标题
CellInfo.prototype.getCaption = function() {
	return this.caption;
}

CellInfo.prototype.getName2 = function() {
	return this.rpt ? (this.rpt.getName ? this.rpt.getName() : this.rpt) + "." + this.getRefName() : "";
}

CellInfo.prototype.getRefName = function() {
	return this.alias || this.dbfld || this.name;
}

CellInfo.prototype.setReport = function(value) {
	this.rpt = value;
}

// 等于getOwner
CellInfo.prototype.getReport = function() {
	return this.rpt;
}

CellInfo.prototype.toString = function() {
	if (this.isFixCell()) {
		return this.caption;
	}
	return this.getName();
}
// ----------------------------------------

/**
 * caption为空，那么就把字段名赋值给caption
 */
function FactField(name, alias, caption, dbfld, exp, group, datatype, isdim, dimname, options, rptname) {
	if (_biInPrototype) return;
	CellInfo.call(this,name, caption, dbfld, alias, datatype);
	this.exp = exp;
	this.group = group;
	this.isdim = isdim;
	this.dimname = dimname;
	this.rptname = rptname;
	this.options = options;
	if (options) {
		this.ppt = new Map(options);
	}
}

_extendClass(FactField, CellInfo, "FactField");

/**
 * 原来是在FactField构造时给每个实例赋值的，没必要给每个实例赋值，改为给原型赋值
 */
FactField.prototype._className = "FactField";

FactField.prototype.clone = function() {
	return new FactField(this.name, this.alias, this.caption, this.dbfld, this.exp, this.group, this.datatype, this.isdim, this.dimname, this.options,
	        this.rptname);
}

/*是否是报表期维*/
FactField.prototype.isBbqDim = function() {
	return this.ppt && parseInt(this.ppt.getValue('dimtype'), 10) == 4;
}

/*是否是上级代码维*/
FactField.prototype.isPidDim = function() {
	return (this.ppt && parseInt(this.ppt.getValue('dimtype'), 10) == 1) || ('upid()'.equalsIgnoreCase(this.exp));
}

FactField.prototype.getRptName = function() {
	return this.rptname;
}

FactField.prototype.getAlias = function() {
	return this.alias;
}

/*返回表名+字段名 xxb.b1， 如果有字段就是表名+字段，否则就是表名+name, 如果是id(), bbq(), upid()就返回该exp*/
FactField.prototype.getName2 = function() {
	return this.rptname + '.' + this.getRefName();
}

FactField.prototype.getFieldName = function() {
	return this.dbfld;
}

/**
 * 一个指标有别名（不是必须的），有名称，也可能有字段名
 * 那么在使用这个表元时如何引用它呢，规则是别名优先、其次是字段名、最后是表元名
 */
FactField.prototype.getRefName = function() {
	return this.alias || this.dbfld || this.name;
}

FactField.prototype.getCaption = function() {
	return this.caption;
}

FactField.prototype.getGroup = function() {
	return this.group;
}

FactField.prototype.getExp = function() {
	return this.exp;
}

/**
 * BI-4547 在BI3.1中主题表结构修改了，可能没有isdim这个了属性了，而判断是否是维，也不全靠这个属性，应该还要看
 *    是否指定了维表
 * @return {}
 */
FactField.prototype.isDimension = function() {
	return this.isdim || (this.dimname != null && this.dimname.length > 0);
}

// 是否是固定表元
FactField.prototype.isFixCell = function() {
	return false;
}

// 是否是数值表元
FactField.prototype.isDataCell = function() {
	return true;
}

// 如果是维，那么返回维名称
FactField.prototype.getDimensionName = function() {
	return this.dimname;
}

FactField.prototype.isCustomBBqDim = function() {
	return this.dimname == '月份' || this.dimname == '季度' || this.dimname == '日期';
}

/*
 * TaskDataInfo对象的js实现。
 */
function TaskDataInfo(owner) {
	this.reportDbInfos = new OrderMap();
	this.owner = owner;
}

TaskDataInfo.prototype._addReportDbInfo = function(rpName, rpDbInfo) {
	this.reportDbInfos.put(rpName, rpDbInfo);
}

// 取得reportDbInfo. 参数最好是报表名。dataInfoplan，如果是空那么就取默认的数据方案
TaskDataInfo.prototype.getReportDbInfo = function(rpName, dataInfoplan) {
	var rpinfo = this.reportDbInfos.get(rpName + dataInfoplan);
	if ((rpinfo == null) || isUndefined(rpinfo) || isUndefined(rpinfo.reportName)) {
		var map = new Map();
		map.put("obj", "reportdbinfo");
		map.put("taskid", this.owner.getTaskId());
		map.put("childobj", "reportinfo");
		map.put("reportname", rpName);
		map.put("datainfoplan", dataInfoplan);
		var result = queryObj("js/querytask.do", map);
		if (result) {
			rpinfo = new ReportDbInfo();
			rpinfo.importFromStr(result);
			rpinfo.reportName = rpName;
			rpinfo.dataInfoplan = dataInfoplan;
			rpinfo.owner = this;
			this._addReportDbInfo(rpinfo.reportName + dataInfoplan, rpinfo);
		}
	}
	return rpinfo;
}

// 移除reportDbInfo. 参数可以中报表名或索引号。
TaskDataInfo.prototype.removeReportDbInfo = function(rpNameOrIdx) {
	this.reportDbInfos.remove(rpNameOrIdx);
}

TaskDataInfo.prototype.clear = function() {
	this.reportDbInfos.clear();
}

// 根据报表名查出它在数据中的索引值。
TaskDataInfo.prototype.indexOf = function(rpName) {
	try {
		for (var i = 0; i < this.getReportCount(); i++) {
			if (this.getReportName(i).toUpperCase() == rpName.toUpperCase())
				return i;
		}
	}
	catch (e) {
	}
	return -1;
}

TaskDataInfo.prototype.getReportCount = function() {
	return this.reportDbInfos.size();
}

TaskDataInfo.prototype.getReportName = function(i) {
	return this.getReportDbInfo(i).reportName;
}

TaskDataInfo.prototype.getTableName = function(i) {
	return this.getReportDbInfo(i).tableName;
}

// ReportDbInfo对象的js实现
function ReportDbInfo(rpNm, tblNm, mid, pid, nname, condition) {
	this.owner = null;
	this.reportName = rpNm;
	this.tableName = tblNm;
	this.mid = mid;
	this.pid = pid;
	this.mname = nname;
	this.condition = condition ? condition : '';

	this.dataInfoplan = '';
	this.dbserver = '';// 该表是属于哪一数据库连接
	this.reportBbqInfo = new ReportBbqInfo(this);
}

ReportDbInfo.prototype.dispose = function() {
	if (this.reportBbqInfo) {
		this.reportBbqInfo.owner = null;
		this.reportBbqInfo = null;
	}
}

/*tablename=;mid=;mname=;bbqkind=;bbqfield=;bbqtype=;frombbqfield=;tobbqfield=*/
ReportDbInfo.prototype.importFromStr = function(str) {
	var map = new Map(str, ';');
	this.tableName = map.getValue('tablename', '');
	this.mid = map.getValue('mid', '');
	this.pid = map.getValue('pid', '');
	this.mname = map.getValue('mname', '');
	this.condition = map.getValue('condition', "");

	this.reportBbqInfo.bbqKind = map.getInt('bbqkind', ReportBbqInfo.BBQKIND_NONE);
	this.reportBbqInfo.bbqField = map.getValue('bbqfield', '');
	this.reportBbqInfo.irptBbqType = map.getValue('bbqtype', '');// 数据期字段类型,是字符型返回C,是日期型返回D
	this.reportBbqInfo.rptBbqType = map.getValue('rptbbqtype', '');
	this.reportBbqInfo.fromBbqField = map.getValue('frombbqfield', '');
	this.reportBbqInfo.toBbqField = map.getValue('tobbqfield', '');
}

ReportDbInfo.prototype.toString = function() {
	return this.reportName;
}

/**
 * 报表期信息对象。
 */
ReportBbqInfo.BBQKIND_NONE = 1; // 没有报表期属性
ReportBbqInfo.BBQKIND_DATE = 2; // 有一个日期型的字段或者字符型的字段，表示报表期
ReportBbqInfo.BBQKIND_FROMTODATE = 4; // 有两个起始日期型的字段表示报表期

function ReportBbqInfo(rpdb) {
	this.reportdbinfo = rpdb;
	this.owner = "";
	this.bbqField = "";
	this.fromBbqField = "";
	this.toBbqField = "";
	this.irptBbqType = "";// 数据期字段类型,是字符型返回C,是日期型返回D
	// rptBbqType:如果bbqKind等于BBQKIND_DATE且表示报表期的字段是一个字符型的字段,
	// 则为BbqType.java中的BBQ_TYPE值的一个
	this.rptBbqType = "";
	this.bbqKind = ReportBbqInfo.BBQKIND_NONE;

	this._isHasBbq = "";
	this._bbqFieldType = "";
	this._allBbqs = null; // BbqGroupedIterator

	this._cachedBbqFieldType = false;
	this._cachedAllBbq = false;
}

ReportBbqInfo.prototype.getBbqFieldType = function() {
	if (this._cachedBbqFieldType)
		return this._bbqFieldType;
	var map = new Map();
	map.put("obj", "reportdbinfo");
	map.put("taskid", this.reportdbinfo.owner.owner.getTaskId());
	map.put("childobj", "bbqfieldtype");
	map.put("reportname", this.reportdbinfo.reportName);
	map.put("datainfoplan", this.reportdbinfo.dataInfoplan);
	var result = queryObj("js/querytask.do", map);
	if (result != null) {
		this._bbqFieldType = result;
	}
	this._cachedBbqFieldType = true;
	return this._bbqFieldType;
}

ReportBbqInfo.prototype.getDefaultBbq = function(onfinish) {
	if (this.defaultBbq)
		return this.defaultBbq;
	var map = new Map();
	map.put("obj", "reportdbinfo");
	map.put("taskid", this.reportdbinfo.owner.owner.getTaskId());
	map.put("childobj", "defaultbbq");
	map.put("reportname", this.reportdbinfo.reportName);
	/**
	 * 在这里会返回默认报表期的名称并以分号分隔
	 * 201010;2010年10月
	 */
	this.defaultBbq = '';
	var result = queryObj("js/querytask.do", map);
	if(result){
		this.defaultBbq = result.split(";")[0];
	}
	return this.defaultBbq;
}

// 实现给irpt.js使用的，其它地方不要使用。
function _editOlapOrQbeRpt_impl(rptidobj) {
	rptidobj = createReportId(rptidobj);
	rptidobj.queryReportInfo(function(obj) {
		        if (obj.isEasyOlap()) {
			        var workspace = getWorkspace();
			        if (workspace) {
				        workspace.showReportEditor(null, obj.rptid, true);
			        }
			        else {
				        window.location.href = "easyolap.do?action=edit&resid=" + obj.rptid;
			        }
			        return;
		        }
		        _downloadReport_impl(obj, function(content, group) {
			                _onfinishDownLoadRpt_editOlapOrQbeRpt_impl(obj, content, group);
		                });
	        });
}

function _onfinishDownLoadRpt_editOlapOrQbeRpt_impl(rptidobj, content, group) {
	if (rptidobj.exttype == 'easyqbe') {
		editEasyQbe(rptidobj.rptid, content, true);
	}
	else if (rptidobj.isOlap()) {
		editOlap(rptidobj.rptid, content, group);
	}
	else if (rptidobj.isQbe()) {
		editQbe(rptidobj.rptid, content, true, group);
	}
}

// 实现给irpt.js使用的，其它地方不要使用。
// 参数rptid可以是Report对象，也可以是字符串，参数type可以传递也可以不传递，传递type可以避免一次服务器请求
function _editReport_impl(rptid) {
	var workspace = getWorkspace();
	/**
	 * irptid可能是字符串
	 * ISSUE:ESENBI-2713 add by RX 2014.11.21
	 * 解决从编辑报表界面计算后的临时表跳转式钻取后的结果表上编辑报表，编辑的是最开始的表的问题
	 * 原因是olddesignid一直在传递中且被优先处理，导致跳转式钻取到的表无法被编辑
	 */
	var srcrptid = rptid.srcRptid;
	var useCurSrcid = false;
	if (typeof(srcrptid) == "string" && srcrptid.length > 1) {
		var sids = srcrptid.split("$");
		if (!!sids && (sids.length > 3) && ("1" == sids[3])) {
			rptid = srcrptid;
			useCurSrcid = true;
		}
	}
	/*
	 * 当前报表形态为1时，认为就是要编辑的报表 by RX 2014.12.02
	 * 解决编辑一个报表时，计算后页面上编辑报表时无法编辑报表的问题
	 */
	if (!useCurSrcid) {
		if (typeof(rptid.olddesignid) == "string" && rptid.olddesignid.length > 1) {// rptid.olddesignid 可能为"", 复现步骤：1.新建一个报表，2.点击计算，3.在结果表上点击编辑
			/**
			 * 一个在服务器上已存在的表，再次被编辑时，被编辑一般还没有保存时计算，就是当作一个新表计算的
			 * 如果用户在计算的结果表上点击“编辑此表”，那么原来我们是把我们在ReportDesignerMgr对象中
			 * 寄存的一个表对象返回，这样是不对的，应该让客户端知道他的正在设计的表的id是什么。
			 */
			rptid = rptid.olddesignid;
		}
		else if (workspace && rptid.getCalcParams && rptid.getCalcParams()) {
			/**
			 * 新建一个表，不保存，直接计算，在计算结果上选择“编辑分析表”，那么之前是有问题的，即又开了一个新的页面去编辑，而不是
			 * 显示之前计算报表时的那个编辑页面，现在修改了，在计算报表时如果是刚新建并且未保存的报表，那么在计算参数中传递：_temp_design_rptid_
			 * 在结果表中，可以通过_temp_design_rptid_在workspace中寻找对应的编辑界面。
			 * TODO 但这并未彻底解决此问题，如果计算后，再保存原表，再到之前的计算结果上选择“编辑分析表”，那就找不到了
			 */
			var _temp_design_rptid_ = rptid.getCalcParams().get("_temp_design_rptid_");
			if (_temp_design_rptid_ && workspace && workspace.showExistedReportEditor(_temp_design_rptid_)) {
				return;
			}
		}
	}

	/**
	 * 如果能在当前的workspace的页面中找到正在被编辑的rptid，那么就直接显示，不必在费事访问服务器了。 
	 */
	if (workspace && !!workspace["showExistedReportEditor"] && workspace.showExistedReportEditor(rptid)) {
		return;
	}
	if (!window["createReportId"]) {//判断不存在引入
		include("ebi/js/funcsindex.js");
	}
	var r = createReportId(rptid);

	if (!r.exttype || r.isResult()) {// 没有exttype或者是结果表，都要请求服务器，结果表要获得源表
		//showProgressDlg("正在装入，请稍等......");
		showProgressDlg(I18N.getString("ebi.user.main.objsatroot.js.str10","正在装入，请稍等......"));
		try {
			r.queryReportInfo(function() {
				        hideProgressDialog();
				        _editReport_withrptType(r);
			        });
		}
		catch (e) {
			hideProgressDialog();
			throw new Error(e);
		}
		return;
	}
	_editReport_withrptType(rptid);
}

function __showEditResource(action, resid) {
	var residobj = createResId(resid);
	if (null == residobj) {
		return;
	}
	var taskid = residobj.getTask().getTaskId();
	//门户上以绝对路径访问
	var url = sys.getContextPath() + action + "?action=edit&taskid=" + taskid + "&resid=" + resid;
	var workspace = getWorkspace();
	if (workspace) {
		workspace.showURL(url);
	}
	else {
		window.location.href = url;
	}
}

/**
 * 根据报表id来选择编辑的方式
 * @param {str,ReportId} rptid
 */
function _editReport_withrptType(rptid) {
	if (!rptid)
		return;
	if (typeof(rptid) == "string")
		rptid = createReportId(rptid);
	/*参数rptid可能就是ReportId对象*/
	if (!rptid || typeof(rptid.queryReportInfo) != "function")
		return;
	var rptidobj = rptid;
	/*获取对应的资源id*/
	var resid = rptid.getRid();

	// 临时计算结果表或者已保存的结果表
	if (rptidobj.isResult()) {
		rptidobj.queryReportInfo();
		/*
		 * ISSUE:ESENBI-1982 add by RX 2014.10.14
		 * 解决逐级式钻取后的钻取结果表页面上编辑分析表提示空指针错误的问题
		 * zq钻取时获取到的srcrptid中的报表形态为4，要转为获取对应源分析表ID才能正确编辑
		 */
		var srcrptid = rptidobj.srcRptid;
		var sids = srcrptid.split("$");
		if ("1" == sids[3]) {
			_editReport_withrptType(srcrptid, rptidobj.srcRptType);
			return;
		}
		else {
			/*
			 * 当右键新建报表，选择的是继续上次编辑的表，那么计算出的临时结果表存在问题originalsrcid为空
			 * 此时使用resid，但也存在问题跳转的编辑界面不是原先的编辑界面。
			 */
			resid = rptidobj.originalsrcid || resid;
		}
	}

	/**
	 * 主题表编辑做了修改，之前的action去掉了，现在统一用ActionEditDWSubject.java
	 */
	if (rptidobj.isFact()) {
		__showEditResource("createfact.do", resid);
		return;
	}

	var workspace = getWorkspace();
	if (rptidobj.isRptTpl()) {
		if (workspace && workspace.showExistedReportEditor(resid)) {
			return;
		}
		__showEditResource("editrpttpl.do", resid);
		return;
	}
	
	if (rptidobj.isRptOlap()) {
		__showEditResource("easyolap.do", resid);
		return;
	}
	// 即席分析
	if (rptidobj.isQbe() || rptidobj.isOlap()) {// 是easyolap、olap、qbe、easyqbe
		return _editOlapOrQbeRpt_impl(rptid);
	}

	/**
	 * 我的收藏夹下存储的报表，可能带中文路径和名称，这里要对rptid进行编码
	 */
	
	if (workspace) {
		workspace.showReportEditor(null, resid);
	}
	else {
		var url = "editrpt.do?action=edit&resid=" + escapeURIComponent(resid) + "&escape=true";
		window.location.href = url;
	}
}

/**
 * 此函数像服务器保存一个草稿
 * key :草稿的名称
 * content : 草稿内容，此对象应该含有下列方法：
 *   export2str():导出草稿内容为一个字符串
 *   getCaption():获取草稿标题
 * onfinish :保存成功后回调
 * params ：备用，是个map
 **/
function _saveDraft_impl(key, content, onfinish, params, silent/*是否不显示进度提示框*/) {
	/**
	 * ISSUE:BI-3900 保存的太快会报Can't execute code from a freed script
	 * 应该不是保存的太快，而是正在处理自动保存草稿时，编辑页面恰好给关闭了，脚本执行到此处时，由于关闭了编辑页面将资源都释放了，这里就会出现“不能执行已释放的脚本”异常
	 * 所以这里要对编辑页中的插件对象进行判断，不存在时就不必向下执行了。
	 * --20100910 by cjb
	 */
	if (!content._activeX)
		return;
	if (!silent)
		//showProgressDlg("正在保存，请稍等......");
	showProgressDlg(I18N.getString("ebi.user.main.objsatroot.js.str11","正在保存，请稍等......"));
	var mp = new Map();
	mp.put("action", "savedraft");
	mp.put("key", key);
	// 保存草稿之前不需要结束当前编辑状态，否则在报表模板中如果编辑一个表元，当内容还没有输入完成就可能因为保存草稿而强行退出编辑了
	mp.put("content", content.export2str(true));
	mp.put("caption", content.getCaption());
	if (params) {
		mp.putMap(params);
	}
	QueryObj.create("js/reqmgr.do", mp, function(query, userdata) {
		        try {
			        query.checkResult();
			        if (!silent)
				       // hideProgressDialogWithComplete(1000, "成功保存为草稿！");// from util.js
			        hideProgressDialogWithComplete(1000, I18N.getString("ebi.user.main.objsatroot.js.str12","成功保存为草稿！"));// from util.js
				/**
			     * ISSUE:BI-3900 保存的太快会报Can't execute code from a freed script
			     * 有可能在执行这个地方的时候，编辑报表的页面被关闭了，如果这里不对content._activeX进行一下判断，
			     * 则在执行回调时就会出现上面的异常。
			     * --20100919 by cjb
			     */
			    if (content._activeX && userdata) {
				        CallBackFunc.doCallBack(userdata, query.getDetail());
			        }
		        }
		        catch (ex) {
			        hideProgressDialog();
			        throwError(ex.message);
		        }
	        }, onfinish);
}

/**
 * 实现给irpt.js使用
 * 更多的参数放在params中,params可以为Map,也可以为string,如:a1=1;a2=2
 */
function _downloadReport_impl(rptid, onfinish, base64encode, params) {
	//showProgressDlg("正在载入，请稍等......");
	showProgressDlg(I18N.getString("ebi.user.main.objsatroot.js.str13","正在载入，请稍等......"));
	/**
	 * rptid有可能是中文，那么跟在url后面就必须经过编码
	 */
	var mp = new Map();
	mp.put("action", "downloadrpt");
	mp.put("rptid", rptid);
	if (base64encode) {
		mp.put("base64", true);
	}
	if (params) {
		if (typeof(params) == "string") {
			mp.merge(params);
		}
		else {
			mp.putMap(params);
		}
	}
	QueryObj.create("js/reqmgr.do", mp, _downloadReportOnFinish, onfinish);
}

function _downloadReportOnFinish(query, userdata) {
	hideProgressDialog();// from util.js
	query.checkResult();
	var dtl = query.getDetail();
	var grp = query.getMessage();
	if (userdata) {
		CallBackFunc.doCallBack(userdata, dtl, grp);
	}
}

/*下载系统函数onfinish(xml)，xml是一个对象
系统函数"ebi/funclist.xml"
系统算子"ebi/sysarith.xml"
*/
function _downloadSysFuncs_impl(path, onfinish) {
	QueryObj.create(path, null, _downloadSysFuncs, onfinish);
}

function _downloadSysFuncs(query, onfinish) {
	var xml = query.getResponseXML();
	if (onfinish) {
		CallBackFunc.doCallBack(onfinish, xml);
	}
}

//与公共平台的方法重名,会覆盖掉,去掉
///*每个一段时间ping一次服务器使客户端不超时*/
//function startPingServer(timeout) {
//	var tp = getRootWindow();
//	if (tp._sanlink_pingserver_timerid) {
//		return;
//	}
//	if (!timeout) {
//		/**
//		 * 此处的默认时间确定为1分钟，可以修改的更频繁，但不能改的太慢，因为：
//		 *   1：LoginAction中renewEditLock，的缺省时间是3分钟
//		 *   2：SessionCounter类的线程轮询时，sesson的无效超时是3分钟
//		 */
//		timeout = 1000 * 1 * 60;
//	}
//	tp._sanlink_pingserver_timerid = tp.setTimeout(_doPingServer, timeout);
//}
//
//function _doPingServer() {
//	window.nextPingTimeout = 0;
//
//	QueryObj.create('js/login.do?action=ping&date='+new Date().getTime(), null, function(queryobj) {
//		        var tp = getRootWindow();
//		        tp._sanlink_pingserver_timerid = null;
//		        queryobj.checkResult();
//		        var txt = queryobj.getDetail();
//		        if (txt == "stopping")
//			        return;
//		        startPingServer(nextPingTimeout);
//
//		        if (txt)
//			        try {
//				        eval(txt);
//			        }
//			        catch (ex) {
//			        };// 执行服务器返回的脚本
//	        });
//}
//
//function stopPingServer() {
//	var tp = getRootWindow();
//	if (tp._sanlink_pingserver_timerid) {
//		tp.clearTimeout(tp._sanlink_pingserver_timerid);
//		tp._sanlink_pingserver_timerid = null;
//	}
//}

function collectRefReport_impl(expzzs, r) {
	var pat = /([a-zA-Z_]\w*\-\>[a-zA-Z_]\w*)|([a-zA-Z_]\w*[.][0-9a-zA-Z_]\w*)/g;
	if (!r)
		r = new Array();
	if (!expzzs)
		return r;

	if (isString(expzzs)) {
		var matchs = expzzs.match(pat);
		for (var i = 0; matchs && i < matchs.length; i++) {
			var item = matchs[i];
			if (!item)
				continue;
			var k = item.indexOf('.');
			if (k == -1) {
				k = item.indexOf('->');
			}
			if (k == -1)
				return r;
			var rpnm = item.substring(0, k).toUpperCase();
			if (r.indexOf(rpnm) == -1) {
				r.push(rpnm);
			}
		}
		return r;
	}

	for (var i = 0; i < expzzs.length; i++) {
		collectRefReport(expzzs[i], r);
	}
	return r;
}

// 返回所有的维字段，如果是多个表，含有相同的维字段，只保留其中一个维字段 onfinish(result)
// 在使用该函数之前，task和report都已经初始化了，只是getTaskDims没有初始化，故在这里使用异步即可。
function collectDimsByRpts_impl(taskid, rpts, onfinish) {
	var map = new Map();// 保存维名称，如果维名称
	//挂在门户下面，getTask一直取不到
	if(!window["getTask"]){
		include("ebi/js/funcsindex.js");
	}
	var tsk = getTask(taskid);
	tsk.getTaskDims(function(dims) {
		        var result = [];
		        for (var i = 0, len = rpts.length; i < len; i++) {
			        var nm = rpts[i];
			        var rpt = tsk.getDataReport(nm);
			        if (rpt) {
				        for (var j = 0, len = rpt.getDimCount(); j < len; j++) {
					        var fld = rpt.getDim(j);
					        var dimnm = fld.getDimensionName();
					        if (dimnm == '$DIM_PID') {
						        dimnm = dims.dimPidName;
					        }
					        else if (dimnm == '$DIM_MID') {
						        dimnm = dims.dimMidName;
					        }
					        if (dimnm) {// 带名称的维
						        if (map.containsKey(dimnm))
							        continue;
						        map.put(dimnm, '');
					        }
					        result.push(fld);
				        }
			        }
		        }
		        onfinish(result);
	        });
}

/*通过行列号获取表元名*/
function getCellNameByRowCol_impl(row, col) {
	var r, c;
	col++;
	if (col < 27) {
		c = String.fromCharCode(col + 64);
	}
	else {
		c = String.fromCharCode((col - 1) / 26 + 64) + String.fromCharCode((col - 1) % 26 + 65);
	}

	row++;
	return "" + c + row;
}

// 返回表元的行列号 返回一个数组，
function getRowFromCellName_impl(clnm) {
	var rowName, row;
	for (var i = 0, len = clnm.length; i < len; i++) {
		if ((clnm.charAt(i) >= '0') && (clnm.charAt(i) <= '9')) {
			rowName = clnm.substring(i, len);
			break;
		}
	}

	row = (parseInt(rowName, 10) - 1);
	return row;
}

function getColFromCellName_impl(clnm) {
	var colName, col;
	for (var i = 0, len = clnm.length; i < len; i++) {
		if ((clnm.charAt(i) >= '0') && (clnm.charAt(i) <= '9')) {
			colName = clnm.substring(0, i);
			break;
		}
	}

	colName = colName.toUpperCase();
	if (colName.length == 1) {
		col = (colName.charCodeAt() - 'A'.charCodeAt());
	}
	else {
		col = (colName.charAt(0).charCodeAt() - 'A'.charCodeAt() + 1) * 26 + colName.charAt(1).charCodeAt() - 'A'.charCodeAt();
	}
	return col;
}

/**
 * by guor 2014/3/31 平台中有ping服务器的功能，BI中删掉
 * @param {} rptid
 */
//startPingServer();

/**
 * 此类描述一个和服务器端AnaObjectId类一样的对象，可能是主题表、报表模板、结果表、即席分析、分析报告、甚至普通文件等
 * 
 * 此对象的id，是包含taskid和类型的长ReportId
 * this.rptid ;
 *
 * 此对象的类型，可选值是服务器端的AnaConst类中的ANATYPE_开头的常量
 *   基本表数据表0xxb,1basic,2planar
 *   固定分析表3
 *   olap表9
 *   qbe表10  
 * public static final int ANATYPE_REPORT = 3;
 * public static final int ANATYPE_QBE = 10;
 * public static final int ANATYPE_EASYQBE = 11;
 * public static final int ANATYPE_DLGOLAP = 9;
 * public static final int ANATYPE_EASYOLAP = 8;
 * public static final int ANATYPE_ANALYSE_WORD = 0x30;
 * public static final int ANATYPE_STATIC_FILE = 0xFFFF;
 * this.anaType;
 *
 * 字符串，表示报表的扩展类型，可能是easyolap
 * this.exttype;
 *
 * 钻取源表的类型
 * this.srcRptType;
 *
 * this.reportName;
 *
 * this.reportCaption;
 *
 * 报表在服务器上的存储状态，可能有如下几种,即服务器上AnaConst类中定义的几个常量
 * public static final int ID_KIND_DATAREPORT = 0;
 * public static final int ID_KIND_ANALYSEREPORT = 1;
 * public static final int ID_KIND_TEMPCALCRESULT = 2;
 * public static final int ID_KIND_CALCRESULT = 3;
 * public static final int ID_KIND_DESIGNING = 4;
 * public static final int ID_KIND_SYSTEMUSE = 5;
 * public static final int ID_KIND_VFS = 6;
 * this.idKind;//请使用相应的isXXXX方法去判断，不要直接引用idKind
 *
 * 是否是一个钻取结果表，如果是一个钻取结果表，那么报表显示界面上的编辑按钮和计算按钮可能不可见
 * this.isDrillResult;
 *
 * 如果是钻取结果表则返回钻取源表的reportid
 * 如果是qbe结果表或olap结果表则返回计算的源表，有可能是一个主题集自带的分析表有可能是用户发布的分析表，也有可能是一个正在设计的报表。
 * this.srcRptid;
 *
 * 是否需要分页浏览条
 * this.needPageNavbar;
 *
 * 如果此表显示的是有数据的表，则此值表示是否还有更多数据在其他的数据页。
 * this.hasMoreData;
 *
 * 共有多少条数据，此值可能没有也可能为-1表示无法估计
 * this.totalDataRowCount;
 *
 * 此页显示了多少条。
 * this.thisPageSize;
 *
 * 返回以逗号分割的主题表的表名，表示所引用的主题表
 * this.refedFacttables;
 *
 * 此页是从第几条开始显示的。
 * this.startIndex;
 * 
 * 返回该表所在的组
 * this.group;
 * 
 * 一个在服务器上已存在的表，再次被编辑时，被编辑一般还没有保存时计算，就是当作一个新表计算的
 * 如果用户在计算的结果表上点击“编辑此表”，那么原来我们是把我们在ReportDesignerMgr对象中
 * 寄存的一个表对象返回，这样是不对的，应该让客户端知道他的正在设计的表的id是什么。
 * this.olddesignid;
 * 
 * 如果是一个计算结果表那么此参数表示，计算此表是所传递的计算参数，一个StringMap对象，以分号分割
 * this.calcparams;
 *
 * 计算此表时的日志信息
 * this.calclogs;
 *
 * String对象 记录报表参数和自定义函数的String对象，可能为空
 * this.reportparams;
 * 
 * 如果此表是一个已保存的计算结果表，那么此属性表示此表的saveid
 * this.saveid;
 * 
 * 如果此表是一个已保存的计算结果表，那么此属性表示此表的源表还有哪些可选的计算结果，此对象是一个
 * BiResource对象数组，具体有哪些属性可参考服务器端类ReqMgr中的listResults方法
 * 此属性需要通过函数listResults初始化
 * this.calcResults;
 */
function ReportId(rptid) {
	if (rptid) {
		this.init(rptid);
	}
}

/**
 * 参见服务器端AnaConst.java中的注释
 */
ReportId.EXTTYPE_REPORT = "rpt";
ReportId.EXTTYPE_QBE = "qbe";
ReportId.EXTTYPE_EASYQBE = "easyqbe";
ReportId.EXTTYPE_DLGOLAP = "olap";
ReportId.EXTTYPE_EASYOLAP = "easyolap";
ReportId.EXTTYPE_WORD = "doc";
ReportId.EXTTYPE_STATIC_FILE = "unknown";
ReportId.EXTTYPE_FACT = "fact";
ReportId.EXTTYPE_VFACT = "vfact";
ReportId.EXTTYPE_RPTTPL = "rpttpl";
ReportId.EXTTYPE_RPTOLAP = "rptolap";
/**
 * 参见服务器端AnaConst.java中的注释
 */
ReportId.ANATYPE_REPORT = 3;
ReportId.ANATYPE_QBE = 10;
ReportId.ANATYPE_EASYQBE = 11;
ReportId.ANATYPE_DLGOLAP = 9;
ReportId.ANATYPE_EASYOLAP = 8;
ReportId.ANATYPE_ANALYSE_WORD = 0x30;
ReportId.ANATYPE_STATIC_FILE = 0xFFFF;

ReportId.ANATYPE_RPTTPL = 4;
ReportId.ANATYPE_RPTOLAP = 5;
/**
 * 解析一个字符串形式的anaid，可以允许带上resourceid前缀，
 * 报表ID的组成形式是： (12 + $ + )taskid + $ + idkind + $ + 报表代号 + . + exttype
 */
ReportId.prototype.init = function(rptid) {
	this.rptid = rptid;
	var ss = rptid;
	var sss = ss.split("$");
	var restp;
	var sss0 = sss[1];
	if ("24" == sss0) {// 我的收藏
		this.reportKind = 1;// ID_KIND_ANALYSEREPORT
		restp = sss0;
		return;
	}
	var k = 0;
	if ("10" == sss0 || "12" == sss0) {// rptid带了资源id头，要兼容这种情况
		restp = sss0;
		k = 2;
	}
	this.taskid = sss[k + 0];
	this.idKind = this.reportKind = parseInt(sss[k + 1], 10);
	this.id = sss[k + 2];
	var i = this.id.indexOf('.');
	if (i >= 0) {
		this.exttype = this.id.substring(i + 1);
		this.id = this.id.substring(0, i);
		this.anaType = this.extType2AnaType(this.exttype);
	}

	if (!restp) {
		// 如果没有restp那么肯定是主题表或分析表（结果），不会是vfs内的表
		// 此处总是给rptid补上资源头，现在各个地方都是含资源头的
		this.rptid = (this.reportKind == 0 ? "EBI$10$" : "EBI$12$") + this.rptid;
	}

}

/**
 * 销毁对象
 */
ReportId.prototype.dispose = function() {
	
};

ReportId.prototype.extType2AnaType = function(exttype) {
	if (ReportId.EXTTYPE_REPORT.equalsIgnoreCase(exttype)) {
		return ReportId.ANATYPE_REPORT;
	}
	else if (ReportId.EXTTYPE_QBE.equalsIgnoreCase(exttype)) {
		return ReportId.ANATYPE_QBE;
	}
	else if (ReportId.EXTTYPE_EASYQBE.equalsIgnoreCase(exttype)) {
		return ReportId.ANATYPE_EASYQBE;
	}
	else if (ReportId.EXTTYPE_DLGOLAP.equalsIgnoreCase(exttype)) {
		return ReportId.ANATYPE_DLGOLAP;
	}
	else if (ReportId.EXTTYPE_EASYOLAP.equalsIgnoreCase(exttype)) {
		return ReportId.ANATYPE_EASYOLAP;
	}
	else if (ReportId.EXTTYPE_WORD.equalsIgnoreCase(exttype)) {
		return ReportId.ANATYPE_ANALYSE_WORD;
	}
	else if (ReportId.EXTTYPE_RPTTPL.equalsIgnoreCase(exttype)) {
		return ReportId.ANATYPE_RPTTPL;
	}
	else if (ReportId.EXTTYPE_RPTOLAP.equalsIgnoreCase(exttype)) {
		return ReportId.ANATYPE_RPTOLAP;
	}
	return;
}

/**
 * 返回报表的资源ID，如果是主题表或者分析表表样以及我的收藏夹的报表返回其实际的资源ID，如果是结果表，一律返回分析表的资源ID
 */
ReportId.prototype.getRid = function() {
	/**
	 * this.reportKind的值写错了，其值应该是报表类型，不是资源类型
	 */
	return this.rptid;// rptid的值应该默认就是兼容rid的格式的
}

/*获得对应的文字串信息*/
ReportId.prototype.getId = function() {
	return this.id;
}

/**
 * 获得计算参数的map，不包含一些内部使用的值
 */
ReportId.prototype.getCalcParams = function() {
	if (!this.calcparams) {
		return;
	}
	var m = new Map();
	m.putMap(this.calcparams);
	m.remove("rptid");
	m.remove("startindex");
	m.remove("rptcontent");
	m.remove("pagesize");
	m.remove("totalDataRowCount");
	m.remove("drillParams");
	/**
	 * zqfilter这些参数也不传递
	 */
	m.remove("@ZQFILTER");
	m.remove("@ZQDIMID");
	m.remove("@ZQDIMLEVEL");
	m.remove("@ZQDIMTXT");
	m.remove("@ZQCELLVL");

	return m;
}

ReportId.prototype.getReportParams = function() {
	return this.reportparams;
}

/*返回对应的主题集的id*/
ReportId.prototype.getTaskId = function() {
	return this.taskid;
}

/*获得对应的报表对象,现只支持数据表*/
ReportId.prototype.getReport = function(ifnotExistsThenThrow) {
	return getTask(this.taskid).getDataReport(this.id, ifnotExistsThenThrow);
}

/**
 * 根据报表id列出其对应的结果集合，其结果是个ResourceId 的数组，ResourceId包含这些属性
 * this.owner         = 刷新该用户的id
 * this.id            = 对应的结果表id，  taskid$3$xxxx
 * this.caption        = caption;      结果表对应的caption   
 * this.startCalcTime = startCalcTime; 该结果表对应的计算时间
 * this.params        = params;        计算该表的参数描述
 * this.saveid        = saveid;        如果是个结果表，则本属性是其saveid
 * onfinish是回调函数，里面参数是一个结果表的数组的数组，数组中的元素是一个ResourceId对象
 */
ReportId.prototype.listResults = function(onfinish) {
	if (this.calcResults) {
		return onfinish(this.calcResults);
	}
	var map = new Map();
	map.put("action", "get");
	map.put("obj", "listresults");
	map.put("rptid", this.rptid);
	this._resultonfinish = onfinish;
	QueryObj.create("js/reqmgr.do", map, this._parseListResultInfos, this);
}

ReportId.prototype._parseListResultInfos = function(q, self) {
	q.checkResult();
	var dom = q.getResponseXML();
	var nodes = dom.getElementsByTagName("treenode");
	var arr = [];
	for (var i = 0, len = nodes.length; i < len; i++) {
		var r = new BiResource();
		r.parseFromXml(nodes[i]);
		arr.push(r);
	}
	self.calcResults = arr;
	self._resultonfinish(arr);
	self._resultonfinish = null;
}

ReportId.prototype.toString = function() {
	return this.rptid;
}

/*是分析表*/
ReportId.prototype.isAnalyse = function() {
	return this.idKind == 1 || // 已保存的分析表或分析报告ReportId.ID_KIND_ANALYSEREPORT
	        this.idKind == 4; // 临时分析表分析报告ID_KIND_DESIGNING
}

/**
 * 此对象是否是一个报表模板
 */
ReportId.prototype.isFormatCal = function() {
	return this.anaType == 3;// ReportId.ANATYPE_REPORT
}

/**
 * 此对象是否是一个EasyOlap分析对象，
 */
ReportId.prototype.isEasyOlap = function() {
	return this.anaType == 8;// ReportId.ANATYPE_EASYOLAP
}

/**
 * 新的报表模板
 */
ReportId.prototype.isRptTpl = function() {
	return this.anaType == 4;// ReportId.ANATYPE_RPTTPL
}

/**
 * 新的easyolap
 */
ReportId.prototype.isRptOlap = function() {
	return this.anaType == 5;// ReportId.ANATYPE_RPTOLAP
}

/**
 * 此对象是否是一个Olap或EasyOlap分析对象
 * 20100921 以前是这样判断是否是olap对象：this.anaType == 9 || this.anaType == 8;// ReportId.ANATYPE_DLGOLAP，ReportId.ANATYPE_EASYOLAP
 * 			在wtap中增加trasolap之后，trasolap的anaType是65535，这样判断就不行了。改成查询的方式。
 * 			在wtap中会把trasolap加到olap后缀数组中去。
 */
ReportId.OLAP_EXT_TYPES=["olap", "easyolap"];
ReportId.prototype.isOlap = function() {
	return ReportId.OLAP_EXT_TYPES.indexOf(this.exttype)!=-1;
}

/**
 * 此对象是否是一个QBE或EasyQBE分析对象，
 */
ReportId.prototype.isQbe = function() {
	return this.anaType == 10 || this.anaType == 11;// ReportId.ANATYPE_QBE，ReportId.ANATYPE_EASYQBE
}

/**
 * 此对象是否是一个EasyQBE分析对象，
 */
ReportId.prototype.isEasyQbe = function() {
	return this.anaType == 11;// ReportId.ANATYPE_EASYQBE
}

/**
 * 此对象是否是一个主题表，虚拟主题或物理主题
 * ISSUE:http://192.168.1.200/jira/browse/BI-4308
 * 之前用idKind判断的，但是好像不对
 */
ReportId.prototype.isFact = function() {
	return this.reportKind == 0;// ReportId.ID_KIND_DATAREPORT
}

/**
 * 此对象是否是一个已保存的分析表或分析报告,不带结果的，通常用来计算
 */
ReportId.prototype.isSavedAnalyse = function() {
	return this.idKind == 1;// 已保存的分析表或分析报告，ReportId.ID_KIND_ANALYSEREPORT
}

/**
 * 此对象是否是一个临时的计算结果，例如一个正在编辑的报表的测试计算结果
 */
ReportId.prototype.isTempCalcResult = function() {
	return this.idKind == 2;// 临时计算结果表，ReportId.ID_KIND_TEMPCALCRESULT
}

/**
 * 此对象是否是一个已经保存下来的计算结果，例如用户在门户中计算一个表，那么结果就是
 * @return {}
 */
ReportId.prototype.isSavedCalcResult = function() {
	return this.idKind == 3;// 已保存的报表的结果表，ReportId.ID_KIND_CALCRESULT
}

/**
 * 此对象是否是一个计算结果
 */
ReportId.prototype.isResult = function() {
	return this.isTempCalcResult() || this.isSavedCalcResult();
}

/*返回一个用于计算的id*/
ReportId.prototype.getIdToCalc = function() {
	if (this.isTempCalcResult()) {// 临时计算结果表
		if (this.isDrillResult && this.srcRptType == RT_OLAP && this.isQbe()) {
			return this.rptid;
		}
		return this.srcRptid;
	}
	return this.rptid;
}

/*从服务器上请求此表相关信息,第一次请求后不再请求 callback(reportid),
 *callback可以不传递，如果不传递那么此函数是阻塞式调用queryObj */
ReportId.prototype.queryReportInfo = function(callback) {
	if (this._hasQuery) {
		if (callback)
			callback(this);
		return;
	}

	var self = this;
	var obj = new QueryObj("js/reqmgr.do", "action=get&obj=rptidinfo&rptid=" + this.rptid, callback ? function(query) {
		        _reportidCallBack(self, query, callback);
	        } : null);
	if (!callback) {
		_reportidCallBack(this, obj);
	}
}

function _reportidCallBack(sender, query, callback) {
	try {
		query.checkResult();
	}
	catch (ex) {
		// BI-1708 当获取报表出现异常时，例如报表不存在，应该先关闭进度提示对话框，然后在抛出异常
		hideProgressDialog();
		throwError(ex.message);
	}
	var info = query.getDetail();
	_a1 = sender;
	sender._hasQuery = true;
	eval(info)
	if (callback)
		callback(sender);
}

/*和java端的ReqMgr类交互的计算请求对象*/
function CalcReqTask() {
	if (_biInPrototype)
		return;
	AbstractReqObj.call(this);

	/**
	 * 这里不能直接调用progressDialog()，因为showStatus为被调用多次，如果progressDialog事先没有被初始化
	 * 那么也会在这里可能初始化多次，会导致已终止操作，脚本'xui/ctrls/xdialog.js'加载失败!的异常。以前没有出问题
	 * 是因为进入BI界面时，就已经初始化了progressDialog。
	 */
	progressDialog();
}

_p = _extendClass(CalcReqTask, AbstractReqObj, "CalcReqTask");

/*执行请求，向服务器发送用户设置好的各个参数*/
CalcReqTask.prototype.execute = function() {
	if (!this.isCanceled())
		this.start();
}

/*用于设置计算一个服务器上已存在的表的id*/
CalcReqTask.prototype.setCalcReportId = function(rptid) {
	this.options.put("action", "calc");
	this.options.put("rptid", rptid);
}

/*设置要计算哪个表对象，rpt是rp.js中定义的Report对象, taskid为分析表所属的主题集的id，
 * exttype是报表的扩展类型，在objsatroot.js中的ReportId对象中定义
 用于计算一个客户端新建的表，olap或qbe
 */
CalcReqTask.prototype.setCalcReport = function(taskid, rpt, exttype) {
	this.options.put("action", "calcnew");
	this.options.put("taskid", taskid);
	this.options.put("exttype", exttype);
	this.options.put("rptcontent", (typeof(rpt.export2str) == "function") ? rpt.export2str() : rpt);
}

/*设置钻取参数，钻取参数是此js中定义的类DrillParams的实例，此方法用户drill使用*/
CalcReqTask.prototype.setDrillParams = function(drillParams) {
	this.options.put("action", "drill");
	this.options.put("drillParams", (typeof(drillParams.export2str) == "function") ? drillParams.export2str() : drillParams);
}

/*设置返回结果的每页的行数，如果不设置，则返回所有的结果*/
CalcReqTask.prototype.setResultPageSize = function(value) {
	if (value != null && value != undefined) {
		this.options.put("pagesize", value);
	}
}

/*设置数据总共有多少行
   * 当qbe计算时第一次计算需要求出共有多少行数据，但当查看第2页第3页时应该不需要了，此处
   * 计算参数将第一次qbe引擎返回的总记录数记下在第2页和以后页的时候告知qbe引擎，使其不必多次计算总记录数
   *
   * 如果totalDataRowCount为－1表示不知道共有多少行
*/
CalcReqTask.prototype.setTotalDataRowCount = function(value) {
	if (value != null && value != undefined) {
		this.options.put("totalDataRowCount", value);
	}
}

/*设置返回的结果是从第几条开始，从0开始*/
CalcReqTask.prototype.setResultStartIndex = function(value) {
	if (value != null && value != undefined) {
		this.options.put("startindex", value);
	}
}

/*设置结果显示在哪个页面，有如下可能值：
  self：替换自己的页面
  blank：显示在一个新的页面里。
  缺省：如果有workspace则显示在workspace里，否则替换自己的页面
*/
CalcReqTask.prototype.setResultTarget = function(value) {
	this.resultTarget = value;
}

/*向服务器发送计算请求*/
CalcReqTask.prototype._sendQuery = function() {
	this.queryserver("js/reqmgr.do", this.options);
}

/*实现父类需要的方法*/
CalcReqTask.prototype.asynStart = function() {
	this._sendQuery();
};

/**
 * 处理轮询时服务器返回数据 
 */
CalcReqTask.prototype.showStatus = function() {
	progressDialog().setLogs(this.status.getLogs());
};

/**
 * 重写父类方法，根据是否有客户端转换invoke方法进行判断<br>
 * 20100308 增加对轮询返回信息中的invoke和invokeData两个键的额外检查，服务器可能在处理到flash统计
 * 图后，通过这两个键来向客户端发出flash转换为图片的指令，invoke是一个bool值，用于判断是否
 * 发送出了指令，而invokeData则是一个json字符串，内部封装了一个map，该map以代表一个表头的
 * guid为key，以该表头的统计图信息map为值，包括如下信息： swfurl:flash统计图的swf路径
 * xmlurl:flash统计图的xml访问路径，访问一次后会自动删除 chartwidth:统计图宽度 chartheight:统计图高度
 * charttype：统计图类型
 */
CalcReqTask.prototype.querynext = function() {
	var status = this.status;
	var opts = status.getOptions();
	if (opts.getBool("invoke")||opts.getBool("invokeditu")) {
		if (opts.getBool("invoke")) {
			var data = opts.getValue("invokeData");
			if (data != null) {
				var json = JSON.parse(data);
				/*
				 * 泛鹏分析报告统计图计算效率问题改进 20170307
				 */
//				var converter = this.getFlashConverter();
//				for ( var i = 0; i < json.length; i++) {
//					var obj = json[i];
//					converter.convertFlashToImg(obj, this._uploadConvertedImages,
//							this);
//				}
				var converter = this.getChartToWord();
				converter.convertFlashToImg(json, this._uploadConvertedImages,
						this);
			}
		}
		if(opts.getBool("invokeditu")){
			var dataditu =opts.getValue("invokedituData");
			if(dataditu != null){
				var jsonditu = JSON.parse(dataditu);
				var converter = this.getFlashConverterDitu();
				for (var i = 0; i < jsonditu.length; i++) {
					var obj = jsonditu[i];
					converter.convertFlashToImg(obj, this._uploadConvertedImages4ditu, this);
				}
			}
		}
	}
	/*
	 * add by szy at 20140923，解决在门户中timeout不执行的bug，无论是否有flash统计图，都执行基类的方法。
	 */
	AbstractReqObj.prototype.querynext.call(this);
}

CalcReqTask.prototype.getChartToWord = function() {
	return getObjectFromRoot("ChartToWord", "_ChartToWord", true, "ebi/user/main/charttoword.js");
}

CalcReqTask.prototype.getFlashConverter = function() {
	return getObjectFromRoot("FlashConverter", "_FlashConverter", true, "ebi/js/chart/flashconverter.js");
}

CalcReqTask.prototype.getFlashConverterDitu = function() {
	return getObjectFromRoot("FlashConverterDitu", "_FlashConverterDitu", true, "ebi/js/chart/flashconverterditu.js");
}
/**
 * 将通过ChartUtil中转换flash统计图得到的图片base64数据上传到ReqMgr的uploadimg处理
 * 
 * @param {CalcReqTask} self
 * @param {string} chartid 对应word中的相应Shape的name
 * @param {string} charttype 统计图类型，服务器端可能需要根据不同类型对imgcontent进行解码
 * @param {string} chartwidth 主要是因为fusionchart解码时需要高度宽度值
 * @param {string} chartheight
 * @param {string} imgcontent base64的图片数据
 */
CalcReqTask.prototype._uploadConvertedImages = function(self, chartid, charttype, chartwidth, chartheight, imagedata, success) {
	// 处理转换结束后的图片数据，并将之上传到服务器中
	var params = new Map();
	params.setValue("id", self.getId());
	params.setValue("action", "uploadimg");
	if (success) {
		params.setValue("imagedata", imagedata);
	}
	else {
		// alert("failed");
	}
	params.setValue("charttype", charttype);
	params.setValue("chartid", chartid);
	params.setValue("chartwidth", chartwidth);
	params.setValue("chartheight", chartheight);
	self.asynQuery(params);
}
CalcReqTask.prototype._uploadConvertedImages4ditu = function(self, obj, imagedata, success) {
	// 处理转换结束后的图片数据，并将之上传到服务器中
	var params = new Map();
	params.setValue("id", self.getId());
	params.setValue("action", "uploadimgditu");
	if (success) {
		params.setValue("imagedata", imagedata);
	}
	params.setValue("dituid", obj.dituid);
	params.setValue("dituwidth", obj.dituwidth);
	params.setValue("dituheight", obj.dituheight);
	self.asynQuery(params);
}
/* 实现父类需要的方法 */
CalcReqTask.prototype.asynQuery = function(params) {
	this.queryserver("js/reqmgr.do", params);
}

/* 重载父类的方法，不显示由cancel触发的异常 */
CalcReqTask.prototype.showError = function(e) {
	if (!this.isCanceled())
		AbstractReqObj.prototype.showError.call(this, e);
};

CalcReqTask.prototype.cancel = function() {
	this._onfinish = null;
	this._onerror = null;/*如果不设置为null,那么有可能还触发_onerror，因为取消的时候也许当前用户已经注销了*/
	this.resetCalcStatus();
	AbstractReqObj.prototype.cancel.call(this);
};

/*实现父类需要的方法*/
CalcReqTask.prototype.finish = function() {
	var onfinish = this._onfinish;
	if (onfinish) {
		this._onfinish = null;// 去掉引用
		CallBackFunc.doCallBack(onfinish, this.status);
	}
	if (this.status.isFinished()) {
		if (this.status.resultIsReport() && !this.status.isCanceled()) {
			this._showCalcResultReport();
		}
	}
	this.resetCalcStatus();
	//如果计算时间足够短 ，那么这个等待时间是留给页面加载过程的时间
	hideProgressDialog(500);// from util.js
};

/*设置计算完毕后的回调事件,回调的方式是v(status) ,参数status表示计算的状态,
 是xmlhttp.js中的ReqObjExeStatus类的实例,status.getId()返回的是计算后的表的id*/
CalcReqTask.prototype.setOnFinish = function(v) {
	this._onfinish = v
}

// 哪个window对象触发的计算
CalcReqTask.prototype.setCallerWindow = function(v) {
	this._callerWindow = v;
}

/*显示计算的结果，如果结果是报表的话*/
CalcReqTask.prototype._showCalcResultReport = function() {
	// 服务端可以改变target
	var opt = this.status.getOptions();
	var target = opt.getValue("target");
	if (!target)
		target = this.resultTarget;
	showReportInTarget_impl(this.status.getId(), target, this._callerWindow, opt.getValue("container"), this.status);
}

CalcReqTask.prototype.setOnError = function(v) {
	this._onerror = v;
}

/**
 * 重置计算状态，这个状态表示当前报表已开始了计算，但还没完成
 * 在正常计算、异常结束计算和已取消计算的情况下调用
 */
CalcReqTask.prototype.resetCalcStatus = function() {
	if (!!this._callerWindow && !!this._callerWindow["_init_start_calc"]
			&& typeof this._callerWindow["_init_start_calc"] === "function") {
		this._callerWindow["_init_start_calc"]();
	}
	if (!!this._callerWindow && !!this._callerWindow["_init_zself_flag"]
			&& typeof this._callerWindow["_init_zself_flag"] === "function") {
		this._callerWindow["_init_zself_flag"]();
	}
};

CalcReqTask.prototype.finishWithError = function(e) {
	var onerror = this._onerror;
	if (onerror) {
		this._onerror = null;
		CallBackFunc.doCallBack(onerror, this.status);
	}
	this.resetCalcStatus();
	hideProgressDialog();// from util.js
}

/**
 * 重载父类的方法，调节每次询问的时间，对于报表计算，刚开始询问的非常频繁这是希望能快点打开计算结果
 * 然后越来越慢，最后最慢2秒钟一次
 */
CalcReqTask.prototype.getTimeInterval = function() {
	if (typeof(this.timeInterval) == "undefined")
		this.timeInterval = 200;
	if (this.timeInterval >= 300 && this.timeInterval < 1000 * 2) {
		this.timeInterval += 200;
	}
	else {
		this.timeInterval += 10;
	}
	return this.timeInterval;
}

// 计算表,rptid报表id，根据报表ID计算该表
function calcByReportId_impl(rptid, onfinishORresulttarget, wnd) {
	var rptidinfo = createReportId(rptid);
	rptidinfo.queryReportInfo(function(reportid) {
		showAdvanceCalcDialog(wnd, reportid.getReportParams(),
				function(params) {
					/*
					 * ISSUE:ESENBI-2685 add by RX 2014.10.20
					 * 解决编辑带分支的表后不关闭编辑页面在结果页面计算无法按设置的分支计算
					 */
//					if (wnd.g_rptpage && wnd.g_rptpage.getCalcInfo) {
//						var preparam = wnd.g_rptpage.getCalcInfo().calcparams;
//						if (null != preparam) {
//							var olddesignid = preparam.getValue("olddesignid");
//							if (!!olddesignid) {
//								params.put("olddesignid", olddesignid);
//							}
//						}
//					}
					/*
					 * ESENBI-3174 add by kangx 2014-12-9
					 * 统一参数合并，上次计算的参数与当前参数合并，覆盖旧有参数 
					 */
					if (wnd.g_rptpage && wnd.g_rptpage.calcParam) {
						var params2 = wnd.g_rptpage.calcParam.getValuesAsMap();
						params.putMap(params2);
						params = wnd.g_rptpage.mergeOldCalcParam(params)
					}
					/**
					 * ESENBI-6270 edit by zqj 2016.01.26
					 * 如果是word分析报告，将olddesignid（原表id）也放入参数中，后台记日志时会使用
					 */
					if(wnd.g_rptpage && wnd.g_rptpage.calcInfo.calcparams) {
						var calcparams = wnd.g_rptpage.calcInfo.calcparams;
						var exttype = calcparams.get("exttype");
						var olddesignid = calcparams.get("olddesignid");
						if(exttype == "doc" && olddesignid) {
							params.put("olddesignid", olddesignid);
							params.put("exttype", exttype);
						}
					}
					
					fxbCalc_impl(reportid, 0, null, onfinishORresulttarget,
							null, params, wnd);
				}, reportid.rptid, reportid.getTaskId());
	});
}

// 此函数实现给irpt.js使用，其它地方不要调用
function fxbCalc_impl(rptid, startindex, pagesize, resultTarget, totalDataRowCount, ppt, wnd) {
	if (!wnd._init_start_calc) {
		wnd._init_start_calc = function() {
			wnd["_currentreportiscalcing"] = null;
		};
	}
	if (!wnd["_currentreportiscalcing"]) {
		wnd["_currentreportiscalcing"] = true;
		wnd.addEventListener ? wnd.addEventListener("error",
				wnd._init_start_calc, false) : wnd.attachEvent("onerror",
						wnd._init_start_calc);
	}
	sys.lib.include("ebi/js/funcsindex.js");
	var crt = createCalcReqTask();
	if (wnd)
		crt.setCallerWindow(wnd);
	showCalcProgressDlg(null, crt.cancel.bind(crt));
	
	if (typeof(startindex) == "number") {
		crt.setResultStartIndex(startindex);
	}
	if (typeof(pagesize) == "number") {
		crt.setResultPageSize(pagesize);
	}
	crt.setTotalDataRowCount(totalDataRowCount);
	if (typeof(resultTarget) == 'function') {
		crt.setResultTarget('none');
		crt.setOnFinish(resultTarget);
	}
	else {
		crt.setResultTarget(resultTarget);
	}
	crt.addProperties(ppt);
	/*
	 * CHG by RX 2015.08.06
	 * 将这句从很上面的地方挪到这里来了
	 * 解决分析组件在报表内使用了报表参数控件，修改参数值触发自动计算时，参数中的action=calcExist覆盖了
	 * 正常提交计算的action=calc，导致计算不能完成并报错的问题
	 */
	crt.setCalcReportId(rptid);
	crt.execute();
}

/*
 * 将指定的id的报表按target的指示显示出来，target可能为null
 * status {util.js#ReqObjExeStatus} 计算返回的状态
 */
function showReportInTarget_impl(rptid, target, wnd, container, status) {
	wnd = wnd ? wnd : window;
	if (target == "none") {
	}
	else if (target == "self") {
		this._showResultInSelf(rptid, wnd);
	}
	else if (target == "parent") {
		this._showResultInParent(rptid, wnd);
	}
	else if (target == "blank") {
		this._showResultInBlank(rptid);
	}
	else if (target == "workspaceorblank") {
		if (!_showReportInWorkspace(rptid)) {
			this._showResultInBlank(rptid);
		}
	}
	else if (target == "inside") {
		//add by zhuzht ESENBI-4274计算信息
		updateContainerCalcInfo(status, wnd,I18N.getString("ebi.user.main.objsatroot.js.str29", "----钻取计算信息：----"));
		this._showResultInside(rptid, wnd);
	}
	else if (target == "container"){
		if(container){
			var containers = container.split(";");
			/**
			 * add by xujw 2015年3月11日16:59:27
			 * ESENBI-4403  启用了自定义滑动轴自动播放功能的报表关闭后提示：无法获取属性"calcInfo"的值: 对象为 null 或未定义
			 * 原因是：自动计算和手工点击计算。同时操作wnd.g_rptpage这个对象导致的问题。
			 * 
			 * 这里简单处理如果 wnd.g_rptpage对象找不到就直接返回，不然下面也会报错。
			 */
			if(!wnd.g_rptpage) {
				return;
			}
			
			var gridIds = wnd.g_rptpage.calcInfo.gridids;
			var ids = gridIds?gridIds.split(";"):[];
			for(var i = 0, len = containers.length - 2; i <= len; i++){
				if(containers[i]){
//					this._showResultInContainer(rptid, containers[i], wnd, i == len ?"true":"false", ids);
					//之前不明白为什么要传false参数，导致有些热区参数没有显示
					this._showResultInContainer(rptid, containers[i], wnd, "true", ids);
				}
			}
			/*
			 * ISSUE:ESENBI-4162 imp by RX 2015.01.14
			 * 支持热区式钻取时，热区式计算SQL记录到详细信息（日志）内
			 */
			updateContainerCalcInfo(status, wnd);
			wnd.isZcalcOver = true;
			//ESENBI-4443 解决热区式多次调用问题
			wnd._zcalcn(wnd._zcalc4params, null, true);
		}
	}
	else if (!target) {
		if (!_showReportInWorkspace(rptid)) {
			this._showResultInSelf(rptid, wnd);
		}
	}
}

/*
 * 更新页面记录的计算日志信息
 * 用于热区式钻取等，可能需要将计算结果（主要是SQL语句）显示到结果页面“详细信息”（或“日志”）中的情形
 * @param status {util.js#ReqObjExeStatus} 计算返回状态信息
 * @param wnd 原报表计算结果页面
 */
function updateContainerCalcInfo(status, wnd,zcalcmarkstr) {
	if (!status || !status["getLogs"]) {
		return;
	}
	if (!wnd || !wnd.g_rptpage || !wnd.g_rptpage.calcInfo) {
		return;
	}
	var calclog = wnd.g_rptpage.calcInfo.calclogs;
	if (!calclog) {
		return;
	}
	var zcalclog = status.getLogs();
	if (!zcalclog || !isString(zcalclog) || (zcalclog.length < 1)) {
		return;
	}
	var zcalcMarkStr = zcalcmarkstr||I18N.getString("ebi.user.main.objsatroot.js.str28", "----热区计算信息：----");
	var zcalcMarkpos = calclog.indexOf(zcalcMarkStr);
	/*
	 * 热区式钻取时，如果之前记录有热区计算信息，替换热区计算信息，没有就直接添加
	 */
	var remainlog = calclog;
	if (zcalcMarkpos > -1) {
		remainlog = remainlog.substring(0, zcalcMarkpos);
	}
	wnd.g_rptpage.calcInfo.calclogs = remainlog + zcalcMarkStr + "\r\n" + zcalclog;
}

/**
 * 热区钻取,替换整个容器，containerid容器id
 */
function _showResultInContainer(resultReportid, containerid, wnd, isrefresh, ids){
	wnd = wnd ? wnd : window;
	function _afterGetHtml(queryobj) {
		//对象不存在，则直接返回
		if (!wnd.g_rptpage) {
			return;
		}
		var html = queryobj.getResponseText();
		if(containerid){
			var gridids = [];
			for (var i = 0; i < ids.length; i++) {
				if(html.indexOf(ids[i]) > -1){
					gridids.push(ids[i]);
				}
			}
			//热区式问题修改 add by kangx 2014-8-14
			//提取script脚本
			var _exp = /<script.*?>[\s\S]*?<\/script>/gi;
			var _scripts = html.match(_exp);
			//html = html.replace(_exp, "");
			
			var containerDom = wnd.document.getElementById(containerid);
			//containerDom.style.visibility = "hidden";
			//ESENBI-2754，ie8以下浏览器必须保证append的dom是同一个文档流的，否则报 ‘参数错误’
			var tempDiv = wnd.document.createElement("div");
			//ESENBI-4446, ie8一下浏览器如果innerHTML包含了style等样式会丢失,通过以下方式使之起效
			var isiestyle = isie && ieVersion <= 8;
			if (isiestyle) {
				html = "<span style=display:none>MSIE</span>" + html;
			}
			tempDiv.innerHTML = html;
			// 完成后删除无关的元素<span>
			if (isiestyle) {
				tempDiv.removeChild(tempDiv.firstChild);
			}
			//tempDiv.innerHTML = html;
			//var parentN = containerDom.parentNode;
			var newConDiv = tempDiv.firstChild;//新的容器
			wnd.g_rptpage.obj_dispose(containerDom, gridids);//释放之前的对象
			//containerDom作为容器可能被外部持有，修改为只替换里面的内容
			containerDom.innerHTML = "";
			//ESENBI-3494 复制样式时不需要复制位置  add by kangx 2014-12-15
			var left = containerDom.style.left;
			var top = containerDom.style.top;
			containerDom.style.cssText += newConDiv.style.cssText;
			containerDom.style.left = left;
			containerDom.style.top = top;
			for (var i = 0, count = newConDiv.childNodes.length; i < count; ++i) {
				containerDom.appendChild(newConDiv.childNodes[0]);
			}
			wnd.g_rptpage.obj_init(containerDom, gridids);//初始化新的对象
			tempDiv = null;
			//执行脚本
			exeDomScriptJs(containerDom, _scripts, wnd);
			/*
			 * add by RX 2015.07.03
			 * 适应华为项目要求热区式钻取后动态调整其热区内DOM节点样式的需求
			 * 允许通过脚本动态指定执行事件
			 * TODO：需要一个更好的机制，目前这个做法，对于其他热区式钻取到这个id的容器也会触发
			 */
			var freshAreaCallbackFunc = containerid + "_fresh_callbackfunc";
			if (wnd[freshAreaCallbackFunc] && isFunction(wnd[freshAreaCallbackFunc])) {
				wnd[freshAreaCallbackFunc]();
			}
		}
	}

	wnd.rptid = resultReportid; /*设置全局的report id*/
	//门户上以绝对路径访问
	new QueryObj(sys.getContextPath() + "showreport.do?onlycontainer=true&resid=" + resultReportid + "&container=" + containerid + "&isrefresh=" + isrefresh, null, _afterGetHtml);
}

/*显示在当前页面，但只替换显示报表的table dom对象，而不像self方式改变window.location*/
function _showResultInside(resultReportid, wnd) {
	wnd = wnd ? wnd : window;
	function _afterGetHtml(queryobj) {
		try {
			var html = queryobj.getResponseText();
			setReportHtml(html, true, wnd);
			/**
			 * 2011-3-2
			 * TODO 为参数平铺临时加入的代码，当报表显示完成以后，在进行回调
			 */
			if(wnd && wnd.reportContentLoadFinished){
				wnd.reportContentLoadFinished();
			}
		} catch(e) {
		//若果出现异常，这里需要判断是不是autoplay函数执行的，如果是，则要取消继续执行
			if (wnd.g_rptpage && wnd.g_rptpage.autoplayer) {
				wnd.g_rptpage.autoplayer.stop();
			}
			throw e;
		}
	}

	wnd.rptid = resultReportid; /*设置全局的report id*/
	var params = new Map();
	params.put("onlyhtml", true);
	params.put("resid", resultReportid);
	params.put("widgetcenter", _getWidgetCenterInfo(wnd));
	//门户上以绝对路径访问
	new QueryObj(sys.getContextPath() +  "showreport.do?", params, _afterGetHtml);
}

/**
 * 获取客户端显示的组件的居中设置的值
 */
function _getWidgetCenterInfo(wnd) {
	wnd = wnd ? wnd : window;
	if (!wnd["widgetAligns"] || wnd.widgetAligns.length <= 0) {
		return "";
	}
	var widgetAligns = wnd.widgetAligns;
	var data = [], widgetalign, doc = wnd.document;
	for (var i = 0, len = widgetAligns.length; i < len; i++) {
		widgetalign = widgetAligns[i];
		var id = widgetalign.id;
		var widgetDom = doc.getElementById(id);
		//如果指定容器，则需要判断当前居中的节点是否是该容器内的，否则不执行居中
		if (!widgetDom) {
			continue;
		}
		var left = parseInt(widgetDom.style.left, 10);
		var top = parseInt(widgetDom.style.top, 10);
		data.push("{id:"+id+",left:"+left+",top:"+top+"}"); 
	}
	if (data.length > 0) {
		return "[" + data.join(",") + "]";
	}
}

/**
 * 获取报表显示区的滚动栏对象，参数p为id="showreport"的DOM
 * 
 * 虽然getElementsByTagName用起来是很方便，但是如果存在很多相同的DOM时将会影响效率，所以这里分别对Ie与Firefox进行处理
 * 报表结构：
 * <div id="showreport" ...>
 *   <style ...>
 *     <div id="RKTIILFW0TYL8NMQTUS42KSUNJUTUIF60" ...>
 *       <div ...>
 *       
 * @param {} p
 * @return {}
 */
function getReportScrollContainer(p) {
	//恢复之前的判断方法，解决折叠式（_zrn）点击时滚动条回到最顶端的问题
	var node;
	for (var i = 0; i < p.childNodes.length; i++) {
		node = p.childNodes[i];
		/*跳过非元素结点*/
		if (node.nodeType != 1) continue;
		/*跳过非表体结点*/
		if (node.tagName.toLowerCase() != "div") continue;
		var firstdiv = getFirstChild(node);
		if (firstdiv && firstdiv.id == "trial") {//排除试用版div
			firstdiv = getNextElementSibling(firstdiv)
		}
		return firstdiv;
	}
	return null;
}

/**
 * 20090610 在2.0中，setReportHtml的策略是保存统计图，在设置innerHTML之后再还原统计图。
 * 					在2.1初期，setReportHtml的策略是只设置innerHTML，让innerHTML中包含统计图。
 * 					现在，还原到2.0的做法。因为2.1的做法有缺陷，造成很多bug。 
 */
// 设置报表内容,html为报表的新内容,isBackupChart是否保留图表
function setReportHtml(html, isBackupChart, wnd) {
	if (!html)
		return false;
	wnd = wnd ? wnd : window;
	var rptContainer = wnd.document.getElementById("showreport");
	if (!rptContainer)
		return false;
	/**
	 * BI-3063 报表中展开某一结点后，滚动栏总是在最上面
	 * 原因：由于展开的内容是通过innerHTML进行输出的，且是整个报表的输出，所以每次展开后将回滚到首行。如想避免，
	 * 则需要改变输出方式为只向当前展开的结点进行输出或者加锚点，整个输出完后通过锚点来重新定位。
	 * 现以最小的修改量方式来解决上述问题：在展开前记录下滚动栏的位置，在innerHTML后再还原滚动栏的位置
	 * 
	 * --20100302 by cjb
	 * 
	 * ××Firefox中不要轻易的通过firstChild来获取第一个DOM结点，因为它返回给你的很有可能是一个文本结点而不是你所期望的DOM
	 * 原先这里想通过firstChild来获取报表的滚动栏DOM，由于报表的输出是格式化的，这样在Firefox中firstChild返回的是一个回车文本结点，
	 * 所以这里还是通过getElementsByTagName方式来返回滚动栏
	 * --20100303
	 */
	var sc = getReportScrollContainer(rptContainer);
	/**
	 * 报表结构：
	 * <div id="showreport" ...>
	 *   <style ...>
	 *     <div id="RKTIILFW0TYL8NMQTUS42KSUNJUTUIF60" ...>
	 *       <div ...>
	 * 
	 * 这里的结构获取方式只是临时的，在未来修改报表输出方式时需要注意修改该处
	 *  add by kangx 2014-5-19 结构已改
	 */
	var ost = sc.scrollTop;
	wnd.g_rptpage.gsctop = wnd.g_rptpage.getScrollContainerV(sc).scrollTop;

	if (wnd.g_rptpage) {//删除旧有的表格对象以及图的引用
		wnd.g_rptpage.obj_dispose(rptContainer, null);
	}
	/*备份统计图*/
	isBackupChart = typeof(isBackupChart) == "boolean" ? isBackupChart : false;
	// TODO,这种机制应该保留,暂且先切断旧有的引用，然后从新初始化 add by kangx 2014-5-19
	//var bakCharts = backupChart(isBackupChart, wnd);

	/*设置html*/
	setHtmlContent(rptContainer, html);

	/*还原统计图*/
	//restoreChart(bakCharts, wnd);

	/*
	 * 20090609 表体变了，表体选择对象需要重新初始化。否则不能框选表元。
	 * 					参考ISSUE:BI-1788  报表展开后，不能选中指标和维来编辑统计图
	 * 20090904 展开之后要删除统计图的热区。
	 * 					展开后的报表不能从统计图上钻取，因为统计图与报表的数据对不上号了。
	 **/
	if (wnd.g_rptpage) {
		//替换之后需要一些对象 重新初始化，包括图以及表格对象
		wnd.g_rptpage.obj_init(rptContainer, null);
		
		//wnd.g_rptpage.chartMng.afterSetReportHtml();
	}
	/**
	 * innerHTML生成好展开后的报表后将还原先前的滚动栏位置
	 */
	var _sc = getReportScrollContainer(rptContainer);
	if (ost > _sc.scrollTop) {
		/*如在展开结点前滚动栏已经滚动了，则在innerHTML后将滚动栏位置还原*/
		setTimeout(function() {
			        /**
			         * 报表结构：
			         * <div id="showreport" ...>
			         *   <style ...>
			         *     <div id="RKTIILFW0TYL8NMQTUS42KSUNJUTUIF60" ...>
			         *       <div ...>
			         * 
			         * 这里的结构获取方式只是临时的，在未来修改报表输出方式时需要注意修改该处
			         * 这里由于通过innerHTML重新构造了报表，此时需要重新获取报表的滚动栏DOM
			         */
			        _sc.scrollTop = ost;
		        }, 0);
	}
	return true;
}
// 保留图表
function backupChart(isAppend2Body, wnd) {
	var charts = [];

	var imgs = wnd.document.getElementsByTagName("img");
	if (imgs)
		pushNodeList(charts, imgs);

	/**
	 * 20090819 在获取所有统计图dom时，要考虑到flash统计图。否则备份时会漏掉它。
	 * 					参考ISSUE:BI-2396 报表模板中有统计图时，zr 钻取展开下级报错
	 */
	/**
	 * BI-8016,ZR函数以后，统计图消失且报表报错，
	 * 原因是BI3.2.1中对Flash统计图的DOM结构发生了变化。
	 * BI3.2.1之前对Flash统计图只生成了<object></object>标签
	 * BI3.2.1之后对Flash统计图生成了<span ischart="true" swf="" src=""></span>这种结构，然后使用AnyChart或FusionChart的js方法去生成了统计图
	 * 所以这个地方保存统计图的时候，不应该去取object，而应该取span，然后判断是否是统计图的DOM节点
	 */
	var flashchartspans = [];
	var spans = wnd.document.getElementsByTagName("span");
	for ( var i = 0; i < spans.length; i++) {
		if ("true" == spans[i].getAttribute("ischart")) {
			flashchartspans.push(spans[i]);
		}
	}
	if (flashchartspans)
		pushNodeList(charts, flashchartspans);
//	var objs = wnd.document.getElementsByTagName("object");
//	if (objs)
//		pushNodeList(charts, objs);

	if (charts.length == 0)
		return [];

	isAppend2Body = typeof(isAppend2Body) == "boolean" ? isAppend2Body : false;
	var tmp;
	var rs = [];
	for (var i = 0; i < charts.length; i++) {
		tmp = charts[i];

		/*排除不是统计图的图片，_getRealChartImg方法在chartutil.js中定义*/
		tmp = wnd._getRealChartParent(tmp);

		if (tmp != null && rs.indexOf(tmp) == -1)
			rs.push(tmp);
	}
	if (isAppend2Body)
		_appendChart2Body(rs, wnd);
	return rs;
}

/*将节点列表加入到数组中。nodelist一般是dom.childNodes，它不是一个普通的数组，将它加入到数组中不能用concat函数。*/
function pushNodeList(array, nodelist) {
	for (var i = 0; i < nodelist.length; i++) {
		array.push(nodelist[i]);
	}
}

// 恢复图表
function restoreChart(p, wnd) {
	if (!p || p.length == 0)
		return false;
	var headContainer = _getReportHeadContainer(wnd);
	if (!headContainer)
		return false;
	var container = wnd.document.getElementById("$Container4BackupChart");
	if (!container) {
		_restoreChart2Container(headContainer, p);
	}
	else {
		_restoreChart2Container(headContainer, container.childNodes);
		wnd.document.body.removeChild(container);
	}
	return true;
}
// 将图表保存到body里
function _appendChart2Body(p, wnd) {
	if (!p || p.length == 0)
		return false;
	var backupContainer = wnd.document.body.appendChild(wnd.document.createElement("div"));
	backupContainer.id = "$Container4BackupChart";
	var tmp;
	for (var i = 0; i < p.length; i++) {
		tmp = p[i];

		backupContainer.appendChild(tmp);
	}

	backupContainer.style.display = "none";
	return true;
}
// 获取报表头的容器
function _getReportHeadContainer(wnd) {
	var rs = wnd.document.getElementsByTagName("div");
	if (!rs || rs.length == 0)
		return null;
	var tmp;
	for (var i = 0; i < rs.length; i++) {
		tmp = rs[i];
		if (tmp.id.lastIndexOf("_head") == -1)
			continue;
		return tmp;
	}
	return null;
}
function _restoreChart2Container(container, p) {
	if (typeof(container) != "object" || !p || p.length == 0)
		return;
	var tmp;

	// p可能是childNodes对象，appendChild会使tmp从p中删除，所以用Array保存节点信息
	var arr = [];
	for (var i = 0; i < p.length; i++) {
		arr.push(p[i]);
	}
	for (var i = 0; i < arr.length; i++) {
		tmp = arr[i];
		container.appendChild(tmp);
	}
}

/**
 * 20081219 在设置报表的html前清理上一个html。
 */
function _beforeSetReportHtml(wnd) {
	if (wnd.ChartUtil._ctrls) {
		wnd.ChartUtil._ctrls.dispose();
		wnd.ChartUtil._ctrls = null;
	}
}

// 重置统计图管理器
function _resetChartManager(wnd) {
	/**
	 * 20081207 ChartManagerInRpSheet对象在本window中不存在，在wnd中。
	 */
	wnd.ChartManagerInRpSheet.setRpsheet_default();
}

/**
 * 在workspace中显示报表
 * @param {str} rptid 报表rid
 * @param {str} pageid 
 * @param {wnd} wnd window对象，在拼接url时读取window对象的url信息,默认为当前window对象
 * @param {func} onfinish 当页面装入完毕后调用此函数，如果页面已经装入，则直接调用onFinish，onFinish可能为
 * 		字符串或函数，回调时传递的参数为对应的页面的iframe中的contentWindow对象
 * @param {str} place 为bottom则显示页面在下面，就算有同名的页面在上面已经显示了，那么也先把他移
 * 				动下来，为aside 表示显示在当前调用函数所在的workspace的对面，其它情况在mainworkspace中显示
 * @return {boolean} 是否成功显示
 */
function _showReportInWorkspace(rptid, pageid, wnd, onfinish, place,actionurl) {
	/*
	 * edit by zhushy 2014.09.02 ISSUE:[报表搜索]ESENBI-1306 
	 * 在根页面上获取当前显示的TAB页里面的workspace对象，要使用getWorkspaceFromRoot函数。
	 */
	var w = getWorkspaceFromRoot();
	if (w && w.showReport2) {
		w.showReport2(rptid, pageid, wnd, onfinish, place,actionurl);
		return true;
	}
	return false;
}

/*显示在本页面*/
function _showResultInSelf(rptid, wnd) {
	wnd = wnd ? wnd : window;
	/**
	 * add by lpa 解决IRPT-22955 00000000000登录门户，打开业务资料库--税收协定下的分析表，点下一页或末页时报错
	 * 当一个报表嵌入到另一个服务器的门户里面，点击表格分页栏进行翻页，由于此时是跨域的，wnd.parent是访问不到的，会报跨域错误
	 * */
//	if (typeof(wnd.parent.rptpage) != "undefined" && typeof(wnd.parent.rptpage.openReport) != "undefined") {
//		wnd.parent.rptpage.openReport(rptid);
//	}
//	else {
		// fixme 处理原来href中含有的参数，如&showtitle=false&showmenu=false
		//add by kangx 先暂时这样处理
		if (rptid && !rptid.endsWith('.rpttpl')) {
			rptid += '.rpttpl';
		}
		//sys.getContextPath(),门户中访问改成绝对路径访问
		//ESENBI-3931 钻取或者服务器端排序等 都继承原先页面的状态  add by kangx 2015-1-6
		var refparam = setParameter("resid", rptid, wnd);
		if (wnd["showparam"]) {
			refparam = setParameterOfUrl("showparams", true, refparam);
		}
		if (wnd["showmenu"] === false) {
			refparam = setParameterOfUrl("showmenu", false, refparam);
		}
		wnd.location.href = sys.getContextPath() + "showreport.do" + refparam;
//	}
}

/*显示在父页面*/
function _showResultInParent(rptid, wnd) {
	wnd = wnd ? wnd : window;
	if (wnd.parent) {
		// fixme 处理原来href中含有的参数，如&showtitle=false&showmenu=false
		wnd.parent.location.href = sys.getContextPath() + "showreport.do" + setParameter("resid", rptid, wnd.parent);
	}
}

/*显示在一个弹出的新页面，在新弹出的页面中只显示数据，菜单不显示*/
function _showResultInBlank(rptid) {
	window.open(sys.getContextPath() + "showreport.do" + setParameter("resid", rptid) + "&showmenu=false");
}

/*显示出计算的等待对话框*/
function showCalcProgressDlg_impl(msg, oncancel) {
	var progressDlg = progressDialog();
	//progressDlg.setMessage(msg ? msg : "正在计算，请稍等......");
	progressDlg.setMessage(msg ? msg : I18N.getString("ebi.user.main.objsatroot.js.str14","正在计算，请稍等......"));
	progressDlg.showUnknownProgress();
	progressDlg.setHasDetailButton(true);
	progressDlg.setLogs("");
	if (oncancel) {
		progressDlg.setCanCancel(true, oncancel);
	}
	else {
		progressDlg.setCanCancel(false);
	}
	if (!progressDlg.isVisible()) {
		progressDlg.show();
	}
	return progressDlg;
}

/**
 * Gis地图管理对象
 */
function GisMapMgr() {
	this.mapInfos = null;
	this.isLoading = false;
}

/**
 * 从服务器获取gis地图信息，获取完成后执行onfinish，如果参数force为真，那么总是从服务器获取信息，否则尽量使用上次获取的信息
 */
GisMapMgr.prototype.loadFromServer = function(onfinish, force) {
	var self = this;

	if (this.mapInfos != null && !force) {
		if (onfinish)
			onfinish(self);
		return;
	}

	this.isLoading = true;

	QueryObj.create("js/gis.do", "action=getallmap", function(q) {
		        q.checkResult();
		        var allMapInfoXml = q.getDetail();
		        var allMapXmlDom = parseXML(allMapInfoXml);
		        var mapNodes = allMapXmlDom.childNodes;
		        self.mapInfos = new Array();
		        for (var i = 0; i < mapNodes.length; i++) {
			        self.mapInfos[i] = new GisMapInfo();
			        self.mapInfos[i].parseXml(mapNodes[i]);
		        }
		        self.isLoading = false;
		        if (onfinish)
			        onfinish(self);
	        });
}

/*返回GisMapInfo对象数组，可能返回空调用此函数前应该先执行loadFromServer，并等loadFromServer完成后再调用此函数*/
GisMapMgr.prototype.getMaps = function() {
	return this.mapInfos;
}

/**
 * 根据地图id，返回GisMapInfo对象，可能为空，必须先调用loadFromServer初始化mapInfos
 * @param {} id
 */
GisMapMgr.prototype.getMap = function(id) {
	if (!this.mapInfos)
		return null;

	for (var i = 0; i < this.mapInfos.length; i++) {
		var mp = this.mapInfos[i];
		if (mp.id == id) {
			return mp;
		}
	}
	return null;
}

/**
 * 一张GIS地图的信息
 */
function GisMapInfo() {
	this.id;
	this.caption;
	this.layerInfos = [];
}

GisMapInfo.prototype.parseXml = function(elem) {
	this.id = elem.getAttribute("id");
	this.caption = elem.getAttribute("caption");

	var nodes = elem.childNodes;
	for (var i = 0; i < nodes.length; i++) {
		var node = nodes[i];
		var layer = new GisMapLayerInfo();
		layer.parseXml(node);
		this.layerInfos.push(layer);
	}
}

/**
 * 返回GisMapLayerInfo数组，可能为空
 */
GisMapInfo.prototype.getLayersInfo = function() {
	return this.layerInfos;
}

/**
 * 返回地图截屏的base64编码信息字符串，如果传递了onfinish函数，那么异步执行并通过onfinish(base64content)回调
 * 否则同步执行，直接返回结果
 */
GisMapInfo.prototype.getScreenshot = function(w, h, onfinish) {
	if (onfinish) {
		QueryObj.create("js/gis.do", "action=getscreenshot&mapid=" + this.id + "&width=" + w + "&height=" + h, function(q) {
			        q.checkResult();
			        var base64content = q.getDetail();
			        onfinish(base64content);
		        });
		return;
	}
	var q = QueryObj.create("js/gis.do", "action=getscreenshot&mapid=" + this.id + "&width=" + w + "&height=" + h);
	q.checkResult();
	return q.getDetail();
}

/**
 * 一个图层的信息
 */
function GisMapLayerInfo() {
	this.id;
	this.caption;
	/**
	 * 是否是数据图层，数据图层才能定义绑定表元，并自定义区块信息
	 */
	this.isdatalayer
}

GisMapLayerInfo.prototype.parseXml = function(elem) {
	this.isdatalayer = elem.getAttribute("mainlayer") == "true";
	this.caption = elem.getAttribute("caption");
	this.id = elem.getAttribute("id");
}

/**
 * "我的收藏"的全局调用代码
 * 此类用于调用"我的收藏"对话框及建立"我的收藏"的管理界面,被初始化一次,存放于根页面
 * "我的收藏"在左树中以树形结构展示
 */
function MyFavourite() {
}

/**
 * 弹出"我的收藏的对话框"
 * id:为待收藏资源的id
 * icon:待收藏资源的图标,不能缺失
 * caption:当资源保存到"我的收藏"之下的时候,需要一个新的名称,如果不存在，则此参数需传入"",此参数不能缺失
 * onfinish:"我的收藏"对话框点击确定时的回调函数
 */
MyFavourite.prototype.showMyfavoriteDlg = function(id, icon, caption, onfinish) {
	//var rootrids = [["26$~/myfavorite", "我的收藏夹", "ebi/images/lefttree/myfavorite.gif"]];
	var rootrids = [["EBI$26$~/myfavorite",I18N.getString("ebi.user.main.objsatroot.js.str16","我的收藏夹"), "ebi/images/lefttree/myfavorite.gif"]];
	var types = "EBI$25$,EBI$26$";
	showSaveAsDlg(function(dlg) {
		    dlg.setRoot(rootrids, types);
		    dlg.setExt(".link");
		    //dlg.addInput("myfavourite_add2myfav", "名称", caption);
		    dlg.addInput("myfavourite_add2myfav",I18N.getString("ebi.user.main.objsatroot.js.str17","名称"), caption);
		    dlg.setOnSave(function(saveasdlg, overrideResid) {
			        var folder = new BiResource(saveasdlg.getFolder());
			        var name = saveasdlg.getInputValue("myfavourite_add2myfav");
			        var override = !!overrideResid;
			        getMyFavorite().addMyFavorite(name, id, icon, folder, override, function(nm) {
				            if (onfinish)
					            onfinish();
				            // 我的收藏在门户上进行添加的时候,需要对门户上的树进行刷新
				            if (typeof(_xTreeBuilder_) != "undefined")
					            _xTreeBuilder_.reload();
				            // 这里需要手动关闭全局保存对话框
				            saveasdlg.close();
			            });
		        });
		    dlg.show();
	    });
}

/**
 * 创建收藏夹下的管理分组
 * 
 * id:当前选中的节点id,新建的管理分组将为当前节点的子节点
 * 
 * onfinish:回调函数
 * 
 */
MyFavourite.prototype.showCreateMyfavoriteFolderDialog = function(id, onfinish) {
	resource_createGroup(id, null, onfinish);
}

/**
 * 把资源加入到收藏夹中
 * @param {} name      待加入收藏夹的资源的名称 
 * @param {} id       待加入资源收藏夹的id 
 * @param {} icon      该资源在收藏夹显示的图标
 * @param {} destObj   加入到哪个收藏夹下(资源id必须为25或26打头,否则错误)
 * onfinish:回调函数，不管创建是否成功此函数都会被回调，回调形式是onfinish(e)，如果有异常，那么参数e表示异常对象，否则e为null表示创建成功
 *   不管创建是否成功此函数都会被回调，这样可以给onfinish一个处理异常的机会
 */
MyFavourite.prototype.addMyFavorite = function(name, id, icon, destObj, isOverride, onfinish) {
	var content = "link="+id;
	if(icon){
		content += "\r\nicon="+icon; 
	}
	resource_createfile(destObj.id, name+".link", content, onfinish,isOverride);
}

/**
 * 收藏夹内部资源的使用和展示
 * userObj:当前所要展示节点的userObj
 * 
 */
MyFavourite.prototype.showMyfavorite = function(userObj) {
	var firstId = userObj.getFirstId();
	var contextPath = sys.getContextPath();
	var url = encodeURI("vfs" + firstId);
	var ext = userObj.ext;
	if (ext == ".link") {
		var obj = createResId(userObj.realuid);
		_showResource(obj);
	}
	else if (MyfavoriteExts.indexOf(ext) >= 0) {
		var ws = getWorkspace();
		if (ws) {
			ws.showURL(contextPath + url);
		}
		else {
			window.open(contextPath + url);
		}
	}
	else if (ext == '.zip') {
		var params = new Map();
		params.put("command", "download");
		params.put("name", userObj.name);
		params.put("destfile", userObj.caption + userObj.ext);
		params.put("type", "zip");
		openIframeUrl("vfs.do", window, params);
	}
	else {
		window.open(contextPath + url);
	}
}

/**
 * 如果当前用户有消息则依次显示当前登录用户的消息提示，参数timeout为延时多少后再执行显示消息提示，如果该参数设置为0时，
 * 则表示是立即显示消息提示，大于0时为延时到设定的时间后再进行提示
 * @param {} timeout
 */
function showUserMessage(timeout) {
	if (timeout > 0) {
		setTimeout(function() {
			        showUserMessage(0)
		        }, timeout);
		return;
	}
	var userprops = getServer().getLogin().getUserProperties();
	if (!userprops || !userprops.ppt)
		return;
	var keys = userprops.ppt.keySet();
	if (!keys || keys.length == 0)
		return;
	var key, msg;
	for (var i = 0; i < keys.length; i++) {
		key = keys[i];
		if (!key.startsWith("msg_"))
			continue;
		msg = userprops.get(key);
		var sfhm = showFloatHintMsg(msg, 1000 * 30, 100, __showUserMessageOnCloseEvent);
		sfhm.__userdata = key;
		sfhm.setTextAlign("center", "middle");
		break;
	}
}

/**
 * 用户消息提示框在关闭时的事件处理
 * @param {} msgdlg
 */
function __showUserMessageOnCloseEvent(msgdlg) {
	var key = msgdlg.__userdata;
	if (!key)
		return;
	getServer().getLogin().removeUserProperty(key);
	QueryObj.create("js/login.do", "action=removeUserProperties&key=" + key);
	showUserMessage(0);
}

// -----------数据期定义-------------------------------
/**
 * 由于数据期js对象经常用到，又和界面无关，故放置在根页面，该对象的创建，可以通过
 * getObjectFromRoot("BbqOption", null, null, "ebi/user/main/objsatroot.js")
 */
var OPTION_INHERITED = 0;// 当前默认数据期，表元特有类型，表示继承父维表元或分析表的数据期条件
var OPTION_ALLBBQ = 1;// 全部数据期，表示没有数据期条件
var OPTION_FIXBBQ = 2;// 选择数据期，从主题表的数据期字段的数据中选择
var OPTION_FROMTO = 3;// 从选择的起始数据期到选择的停止数据期
var OPTION_BBQEXP = 4;// 手动输入的条件表达式
var OPTION_BBQTOP = 5;// 前n期
var OPTION_CUSTOM = 6;// 选择分析与展现下面或者主题集属性中设置的自定义数据期模板
var OPTION_OFTEN = 7;// 从常用数据期条件中选择一个：本期，上期，本旬，本月，本季，本年

var OPTION_INHERITED_STR = 'inherited';
var OPTION_ALLBBQ_STR = 'all';
var OPTION_FIXBBQ_STR = 'in';
var OPTION_FROMTO_STR = 'fromto';
var OPTION_BBQEXP_STR = 'exp';
var OPTION_BBQTOP_STR = 'top';
var OPTION_CUSTOM_STR = 'custom';
var OPTION_OFTEN_STR = 'often';

var BBQ_OPTIONS = [OPTION_INHERITED_STR, OPTION_ALLBBQ_STR, OPTION_FIXBBQ_STR, OPTION_FROMTO_STR, OPTION_BBQEXP_STR, OPTION_BBQTOP_STR, OPTION_CUSTOM_STR, OPTION_OFTEN_STR];
function BbqOption() {
	//整形，表示数据期类型，初始为全部数据期
	this.type = OPTION_ALLBBQ;
}

BbqOption.prototype.dispose = function() {
	this.type = null;
	this.caption = null;
	this.loadError = null;
	this.fromBbq = null;
	this.toBbq = null;
	this.customBbq = null;
	this.customBbqid = null;
	this.customBbqname = null;
	this.customBbqparam = null;
  this.customBbqcaption = null;
  this.fixBbqInit = null;
  this.fixBbq = null;
	this.fixBbqArr = null;
	this.fromCaption = null;
	this.toCaption = null;
	this.bbqExp = null;
	this.bbqTop = null;
	this.oftenType = null;
	this.oftenCaption = null;
	this.someBbq = null;
};

/**
 * bbqoption=
 * inherited
 * all
 * in:xxx,xxx,xxx
 * fromto:2003-3-1,2004-3-1
 * exp:bbq() like '2002%'
 * top:2 [数字表示前几期]
 * custom:12124844223750.27335954471104146,本月,,本月	[id,name,param,caption]
 * often:current [current(本期)，period(上期)，tendays(本旬)，currentmonth(本月)，currentquarter(本季)，currentyear(本年)]
 */
BbqOption.prototype.loadFromStr = function(str) {
	if (!str) {
		this.type = OPTION_INHERITED;
		return;
	}
	var i = str.indexOf(':');
	if (i == -1)
		i = str.indexOf('\t');
	var param = i == -1 ? "" : str.substring(i + 1);
	var pp = i == -1 ? str : str.substring(0, i).trim().toLowerCase();
	option = BBQ_OPTIONS.indexOf(pp);
	this.type = option;
	switch (option) {
		case OPTION_INHERITED :
			break;
		case OPTION_ALLBBQ :
			break;
		case OPTION_FIXBBQ :
			this.setFixedBbq(param);
			break;
		case OPTION_FROMTO :
			this._parseFromTo(param);
			break;
		case OPTION_BBQEXP :
			this.setBbqExp(param);
			break;
		case OPTION_BBQTOP :
			this.setBbqTop(param);
			break;
		case OPTION_CUSTOM :
			this._parseCustomBbq(param);
			break;
		case OPTION_OFTEN :
			this.setOften(param);
			break;
		default :
			/*
			 * edit by zhushy 2014.09.10 ISSUE:[数据期]ESENBI-1691 保存格式有问题的数据期，提示是否重新设置，若选择重新设置，则打开数据期对话框。
			 * 增加解析出错的标志。
			 */
			this.loadError = true;
	}
};

/*xxx [to] xxx*/
BbqOption.prototype._parseFromTo = function(param) {
	var i = param.indexOf(',');
	if (i == -1) {
		return false;
	}
	this.fromBbq = param.substring(0, i);
	this.toBbq = param.substring(i + 1);
}

BbqOption.prototype.export2str = function() {
	switch (this.type) {
		case OPTION_INHERITED :
			return OPTION_INHERITED_STR;
		case OPTION_ALLBBQ :
			return OPTION_ALLBBQ_STR;
		case OPTION_FIXBBQ :
			return OPTION_FIXBBQ_STR + ':' + this.fixBbq;
		case OPTION_FROMTO :
			return OPTION_FROMTO_STR + ':' + this.getFromBbq() + ',' + this.getToBbq();
		case OPTION_BBQEXP :
			return OPTION_BBQEXP_STR + ':' + this.getBbqExp();
		case OPTION_BBQTOP :
			return OPTION_BBQTOP_STR + ':' + this.getBbqTop();
		case OPTION_CUSTOM :
			return OPTION_CUSTOM_STR + ':' + this.customBbqid + "," + this.customBbqname + "," + (this.customBbqparam == null ? "" : this.customBbqparam) + ","
			        + (this.customBbqcaption == null ? "" : this.customBbqcaption);
		case OPTION_OFTEN :
			return OPTION_OFTEN_STR + ':' + this.getOftenType();
		default :
		throwError(I18N.getString("ebi.user.main.objsatroot.js.str19","报表期解析错误:{0}",[op]));
	}
}

BbqOption.prototype.getType = function() {
	return parseInt(this.type, 10);
}

BbqOption.prototype.setType = function(type) {
	this.type = type;
}

/*如果option==BBQ_OPTION_FIXBBQ获得固定报表期的数目*/
BbqOption.prototype.getFixedBbqCount = function() {
	if (!this.fixBbqInit && this.fixBbq) {
		this.fixBbqArr = this.fixBbq.split(",");
	}
	this.fixBbqInit = true;
	return this.fixBbqArr ? this.fixBbqArr.length : 0;
}

/*如果option==BBQ_OPTION_FIXBBQ获得第i个固定报表期，如2004----*/
BbqOption.prototype.getFixedBbq = function(i) {
	if (!this.fixBbqInit && (this.fixBbq != "")) {
		this.fixBbqArr = this.fixBbq.split(",");
	}
	this.fixBbqInit = true;
	return this.fixBbqArr[i];
}

/*abbq在已经选择的报表期中*/
BbqOption.prototype.existsFixedBbq = function(abbq) {
	for (var i = 0, len = this.getFixedBbqCount(); i < len; i++) {
		if (this.fixBbqArr[i] == abbq) {
			return true;
		}
	}
	return false;
}

/**
 * value是类似2004----,2005----,2006---这样的值
 * */
BbqOption.prototype.setFixedBbq = function(value) {
	this.type = OPTION_FIXBBQ;
	this.fixBbq = value;
	this.fixBbqInit = false;
}

/*如果option==OPTION_FROMTO
  则此值表示从哪一期开始*/
BbqOption.prototype.getFromBbq = function() {
	if (this.type == OPTION_FROMTO) {
		return this.fromBbq;
	}
	else {
		return "";
	}
}
BbqOption.prototype._parseCustomBbq = function(param) {
	if (!param)
		return '';
	if (param.indexOf(',') == -1) {
		throwError(I18N.getString("ebi.user.main.objsatroot.js.str20","所选自定义数据期模板的格式不正确\r\n{0}",[param]));
	}
	var infos = param.split(',');
	this.customBbqid = infos[0];
	this.customBbqname = infos[1];
	this.customBbqparam = infos[2];
	this.customBbqcaption = infos[3];
}
BbqOption.prototype.setCustomBbq = function(id, name, caption, param) {
	this.type = OPTION_CUSTOM;
	this.customBbqid = id;
	this.customBbqname = name;
	this.customBbqcaption = caption ? caption : "";
	this.customBbqparam = param ? param : "";
}

BbqOption.prototype.getCustomBbqId = function() {
	if (this.type == OPTION_CUSTOM) {
		return this.customBbqid;
	}
	else {
		return "";
	}
}
BbqOption.prototype.getCustomBbqName = function() {
	if (this.type == OPTION_CUSTOM) {
		return this.customBbqname;
	}
	else {
		return "";
	}
}
BbqOption.prototype.getCustomBbqParam = function() {
	if (this.type == OPTION_CUSTOM) {
		return this.customBbqparam;
	}
	else {
		return "";
	}
}

function formatBbqCaption(bbq) {
	var year = bbq.substring(0, 4);
	var month = bbq.substring(4, 6);
	var day = bbq.substring(6, 8);
	//return year + "年" + parseInt(month.ensureNotStartWith('0')) + "月" + parseInt(day.ensureNotStartWith('0')) + "日";
	return I18N.getString("ebi.user.main.objsatroot.js.str21","{0}年{1}月{2}日",[year,parseInt(month.ensureNotStartWith('0'), 10),parseInt(day.ensureNotStartWith('0'), 10)]);
}

BbqOption.prototype.setFromBbq = function(value, caption) {
	this.type = OPTION_FROMTO;
	this.fromBbq = value ? value : "";
	this.fromCaption = caption ? caption : "";
}

/*对应于getFromBbq*/
BbqOption.prototype.getToBbq = function() {
	if (this.type == OPTION_FROMTO) {
		return this.toBbq;
	}
	else {
		return "";
	}
}
/**
 * 解决：[ESENBI-2470]easyolap中的数据期设置显示的值与设置的值格式不一致
 * add by wuqing, 2014,11,26
 * @returns
 */
BbqOption.prototype.getFromCaption = function() {
	if(this.type == OPTION_FROMTO){
		return this.fromCaption;
	} else {
		return "";
	}
};
BbqOption.prototype.getToCaption = function() {
	if(this.type == OPTION_FROMTO){
		return this.toCaption;
	} else {
		return "";
	}
};
BbqOption.prototype.setToBbq = function(value, caption) {
	this.type = OPTION_FROMTO;
	this.toBbq = value ? value : "";
	this.toCaption = caption ? caption : "";
}

BbqOption.prototype.getBbqExp = function() {
	if (this.type == OPTION_BBQEXP) {
		return this.bbqExp;
	}
	return "";
}

BbqOption.prototype.setBbqExp = function(value) {
	this.bbqExp = value;
	this.type = OPTION_BBQEXP;
}

BbqOption.prototype.getBbqTop = function() {
	if (this.type == OPTION_BBQTOP) {
		return this.bbqTop;
	}
	return "";
}

BbqOption.prototype.setBbqTop = function(value) {
	this.bbqTop = value;
	this.type = OPTION_BBQTOP;
}

BbqOption.prototype.getOftenType = function() {
	if (this.type == OPTION_OFTEN) {
		return this.oftenType;
	}
	return "";
}

BbqOption.prototype.getOftenCaption = function(oftenType) {
	var oftenCaption = "";
	if (typeof(this.oftenCaption) != 'undefined') {
		return this.oftenCaption;
	}else if(!oftenType){
	}else if(oftenType == "current"){
		oftenCaption = "本期";
	}else if(oftenType == "period"){
		oftenCaption = "上期";
	}else if(oftenType == "tendays"){
		oftenCaption = "本旬";
	}else if(oftenType == "currentmonth"){
		oftenCaption = "本月";
	}else if(oftenType == "currentquarter"){
		oftenCaption = "本季";
	}else if(oftenType == "currentyear"){
		oftenCaption = "本年";
	}
	
	if(oftenCaption){
		this.oftenCaption = oftenCaption;
	}
	return oftenCaption;
}

BbqOption.prototype.setOften = function(oftenType, oftenCaption) {
	this.oftenType = oftenType;
	if(typeof(oftenCaption) != 'undefined'){
		this.oftenCaption = oftenCaption;
	}
	this.type = OPTION_OFTEN;
}

/*判断两个报表期是否相等*/
BbqOption.prototype.equals = function(op) {
	if (this.type == op.type) {
		switch (Number(this.type)) {
			case OPTION_INHERITED :
				;
			case OPTION_ALLBBQ :
				return true;
			case OPTION_FIXBBQ : {
				return this.fixBbq == op.fixBbq;
			}
			case OPTION_FROMTO : {
				return (this.fromBbq == op.fromBbq) && (this.toBbq == op.toBbq);
			}
			case OPTION_BBQEXP : {
				return this.bbqExp == op.bbqExp;
			}
			case OPTION_SOMEDAY : {
				return this.someBbq == op.someBbq;
			}
			case OPTION_CUSTOM : {
				return this.customBbq == op.customBbq;
			}
			case OPTION_BBQTOP :
				return this.bbqTop == op.bbqTop;
			case OPTION_OFTEN :
				return this.oftenType == op.oftenType;
			default :
				//throwError("报表期解析错误:" + op);
			throwError(I18N.getString("ebi.user.main.objsatroot.js.str19","报表期解析错误:{0}",[op]));
		}
	}
	return false;
}

/**
 * 由于这里只记录报表期的值，没法根据值反推出对应的caption，而界面上要显示caption
 * 这样导致实现起来比较麻烦，当然通过指定的事实表，和值是可以反推出显示值的。
 * 
 * 在存储数据期时，同步存储数据期的显示名称
 * @return {}
 */
BbqOption.prototype.getCaption = function(){
	if(this.caption){
		return this.caption
	}
	return this.toString();
}

BbqOption.prototype.setCaption = function(cp){
	this.caption = cp;
}

/*taskid, rptnm只是给当前默认期显示用*/
BbqOption.prototype.toString = function() {
	switch (Number(this.type)) {
		case OPTION_INHERITED :
			//return "报表主题集数据期";
			return I18N.getString("ebi.user.main.objsatroot.js.str22","报表主题集数据期");
		case OPTION_ALLBBQ :
			//return "全部数据期";
			return I18N.getString("ebi.user.main.objsatroot.js.str23","全部数据期");
		case OPTION_FIXBBQ : {
			// 选择的数据期，最多只显示3个，后面的'--'不显示
			var str = "";
			for (var i = 0; i < this.getFixedBbqCount() && i < 3; i++) {
				var bbq = this.getFixedBbq(i);
				str += bbq.ensureNotEndWith("-") + ",";
			}
			str = str.ensureNotEndWith(",")
			if (this.getFixedBbqCount() > 3) {
				str = str + " ...";
			}
			return str;
		}
		case OPTION_FROMTO : {
			var pattern1 = /^\d{4}[0-1][0-9][0-3][0-9]$/g;
			var pattern2 = /^\d{4}[0-1][0-9][0-3][0-9]$/g;
			var f = this.fromCaption; 
			var t = this.toCaption;
			
			if (!f && pattern1.test(this.fromBbq)) {
				f = formatBbqCaption(this.fromBbq);
			}
			if (!t && pattern2.test(this.toBbq)) {
				t = formatBbqCaption(this.toBbq);
			}
			
			if(!f){
				/*
				 * hejzh - 2010.10.28：easyolap设置了起止日期，但是显示出来的是：从undefined到xxxx。
				 * 		原因就是这里一处笔误，把this.fromBbq写成了this.formBbq，导致起始数据期文字为undefined
				 * 		额滴神，害我找了半天。。。看来这种相近的单词造成的笔误要特别小心了！！！
				 */
				f = this.fromBbq;
			}
			
			if(!t){
				t = this.toBbq;
			}

			//return "从 " + f + " 到 " + t + " ";
			return I18N.getString("ebi.user.main.objsatroot.js.str24","从 {0}到{1}",[f,t])+" ";
		}
		case OPTION_BBQTOP :{
//			return "前" + this.getBbqTop() + "期";
			return I18N.getString("ebi.user.main.objsatroot.js.str27","前 {0}期",[this.getBbqTop()]);
		}
		case OPTION_BBQEXP : {
			return this.bbqExp;
		}
		case OPTION_OFTEN : {
			return this.getOftenCaption(this.getOftenType());
		}
		case OPTION_CUSTOM : {
			// 如果名称包含参数，则将参数换成实际的值显示
			var bbqcaption = this.customBbqcaption;
			if (bbqcaption == null || bbqcaption.trim().length == 0) {
				bbqcaption = this.customBbqname;
			}
			var param = this.customBbqparam;
			var txt = bbqcaption;
			var splitArr = null;
			if (bbqcaption != null && bbqcaption.length != 0 && param != null && param.length != 0) {
				var sm = new StringMap(param);
				var keys = sm.keySet();
				for (var i = 0; i < keys.length; i++) {
					splitArr = bbqcaption.split(keys[i]);
					txt = splitArr.join(sm.get(keys[i]));
				}
			}
			return txt;
		}
		default :
			//return "未设置数据期";
		return I18N.getString("ebi.user.main.objsatroot.js.str26","未设置数据期");
	}
}
// -----------------------------------------------------------

function ExpChecker(){

}

ExpChecker.prototype.setTaskId=function(v){
  this.taskid = v;
}

ExpChecker.prototype.setCustomvars=function(v){
  this.customvars = v;
}

/*分析一个表达式，zz可以是任何一个表达式，如果zz能简单的在本地分析完成那么此对象将不会请求服务器。*/
ExpChecker.prototype.parseExp=function(zz){
  this.reset();
  if (this.parseSingleCellExp(zz)){//成功分析，zz是一个单指标表达式。
    return;
  }
  
  if (this.parseConstFunc(zz)){//分析常量函数id bbq upid
    return ;
  }
  var params = new Map();
  params.put("action","compileexp");
  params.put("exp",zz);
  if (this.taskid) params.put("taskid",this.taskid);
  if (this.customvars) params.put("vars",this.customvars);
  var vv = queryObj("js/reqmgr.do", params);
  this._load(vv);
}

/**
 * 获取全局另存为对话框
 * @param {func} onfinish onfinish(saveasdlg, overrideResid)
 */
function getSaveAsDlg(onfinish) {
	//edit by zhushy 2014.08.28 ISSUE:[收藏共享]ESENBI-633 修改bi的全局另存为对话框的ID，避免与平台的另存为对话框发生冲突。
	_getCustomDlg("xui/xui.js,xui/uibase.js,xui/ctrls/xcommonctrls.js,xdialog,xtree,ebi/js/biresource.js;ebi/js/rootjs/lessdlgatroot.js", "SaveAsDlg", function(dlg) {
		    // 可以在此重写资源树请求子节点的url
		    dlg.setQueryUrl(null);
		    onfinish(dlg);
	    }, "_SaveAsDlg_bi");
}

/**
 * 获取全局另存为对话框的报表对话框实现
 * @param {} onfinish
 */
function getSaveAsDlg_Rpt_Impl(onfinish) {
	_getCustomDlg("xui/xui.js,xui/uibase.js,xui/ctrls/xcommonctrls.js,xdialog,xtree,ebi/js/biresource.js;ebi/user/reportpagedsn/dlg/saveasdlg.js", "SaveAsDlg_Rpt_Impl", function(dlg) {
		    // 可以在此重写资源树请求子节点的url
		    dlg.setQueryUrl(null);
		    onfinish(dlg);
	    });
}

/*判断该zz是否是常量函数 id() bbq() upid()*/
ExpChecker.prototype.parseConstFunc = function(zz){
  this.reset();
  if(!zz || zz.length ==0 ) return false;
  var s = zz.trim();
  if(s.equalsIgnoreCase('id()') || s.equalsIgnoreCase('bbq()') || s.equalsIgnoreCase('upid()')){
    this.isValid = true;
    this.returnType = 'C';
    this.refVars = null;
    this.isCellExp = false;
    this.dimensionInCell = null;
    this.refDataCells = null;
    this.isConst = false;
    this.constReturn = null;
    return true;
  }else{
    return false;
  }
}

ExpChecker.prototype._isCellName = function(clnm){
  if (!clnm){
    return false;
  }
  var tt = /([a-zA-Z_]\w*\-\>[a-zA-Z_]\w*)|([a-zA-Z_]\w*[.][a-zA-Z_]\w*)/g;
  var r = clnm.match(tt);
  return r && (r[0]==clnm);
}

/*如果给定的zz是一个简单的表元名称那么此函数返回true并将状态初始化到自己内部的变量*/
ExpChecker.prototype.parseSingleCellExp=function(zz){
  this.reset();
  if (!zz || zz.length==0) return false;
  if (!this._isCellName(zz)){
    return false;
  }
  if (!this.taskid){
    return false;
  }
  /*
   * add by zhushy 2014.11.19 ISSUE:[数据期]根据虚拟主题表创建的easyolap挂在门户下面，打开数据期对话框报错，getServer一直取不到。
   */
  if(!window["getServer"]){
	include("ebi/js/funcsindex.js");
  }
  var t = getServer().getTask(this.taskid);
  if (!t) return false;
  
  var clobj = t.getCell(zz);
  if (clobj==null) return false;
  
  this.isValid = true;
  this.returnType = clobj.getDataType();
  this.detailName = clobj.getCaption();
  this.refVars = null;
  this.isCellExp = true;
  this.dimensionInCell = clobj.getDimensionName();
  this.refDataCells = zz;
  this.refDWSubjects = clobj.getRptName();
  this.isConst = false;
  this.constReturn = null;
  return true;
}

ExpChecker.prototype.reset=function(){
  this.isValid = false;
  this.detailName = null;
  this.returnType = null;
  this.refVars = null;
  this.isCellExp = false;
  this.dimensionInCell = null;
  this.refDataCells = null;
  this.isConst = false;
  this.constReturn = null;
  this.refDWSubjects = null;
}

ExpChecker.prototype._load=function(vv){
  var sm = new StringMap(vv,"\r\n");

  //表达式是否合法
  this.isValid = sm.getBool("isValid",false);

  //表达式的返回类型
  this.returnType = sm.getValue("returnType",null);

  //返回引用了的自定义变量的列表，用逗号分割
  this.refVars = sm.getValue("refVars",null);

  //表示表达式是否是只引用了一个cell且没有其他的操作
  this.isCellExp = sm.getBool("isCellExp",false);

  //如果this.isCellExp为真，则此值表示表元对应的代码组名称
  this.dimensionInCell = sm.getValue("dimensionInCell",null);

  //表示所引用的变量表元的列表，用逗号分割，
  this.refDataCells = sm.getValue("refDataCells",null);

  /**
   * 表达式中引用的主题表
   */
  this.refDWSubjects = sm.getValue("refDWSubjects", null);

  this.isConst = sm.getBool("isConst",false);


  this.constReturn = sm.getValue("constReturn",null);
}

/**
 * 设置全局的rspmeditor的参数
 * @param {RspmEditor} editor
 */
function init_rspmeditor_extra(editor) {
}

/**
 * 创建门户编辑对话框的资源树对象
 * 
 * <br>注：WTAP会覆盖根页面的这个函数，使门户对话框的资源树展示WTAP的资源。
 * 
 * @param {} wnd
 * @param {} parentElement
 * @param {} width
 * @param {} height
 * @return {XClientTree} XClientTree对象或其子类对象
 */
function createPortalEditClientTree(wnd, parentElement, width, height) {
	var restree = new ResourceXTree(wnd, parentElement, width, height);
	var xtree = restree.getXTree();
	xtree.enableCtrlSelect(true);
	xtree.enableShiftSelect(true);
	xtree.dontInsertBlankNode = true;
	xtree.setDraggable(true);
	restree.setRoot("EBI$0$server");
	restree.setTypes("EBI$3$,EBI$7$,EBI$11$,EBI$10$,EBI$13$,EBI$12$,EBI$26$,EBI$35$,EBI$25$");
	restree.doExpand();
	return restree;
}
